#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2012 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Extra widgets and fields
"""
from django import conf
from django import forms
from django.conf import settings
from django.contrib.gis.db import models
from django.contrib.gis.geos import fromstr
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

import re

def getMapJS(area_name=''):
    '''Variable initialization for drawing the map
    '''
    # projection, center and bounds definitions
    js = u"var epsg_display_projection = new OpenLayers.Projection('EPSG:%d')"\
         u";\n" % settings.CHIMERE_EPSG_DISPLAY_PROJECTION
    js += u"OpenLayers.ImgPath = '%schimere/img/';\n" % settings.STATIC_URL
    js += u"var epsg_projection = new OpenLayers.Projection('EPSG:%d');\n" % \
                                               settings.CHIMERE_EPSG_PROJECTION
    js += u"var centerLonLat = new OpenLayers.LonLat(%f,"\
          u"%f).transform(epsg_display_projection, epsg_projection);\n" % \
                                               settings.CHIMERE_DEFAULT_CENTER
    js += u"var media_path = '%s';\n" % settings.MEDIA_URL
    js += u"var static_path = '%s';\n" % settings.STATIC_URL
    js += u"var map_layer = %s;\n" % settings.CHIMERE_DEFAULT_MAP_LAYER
    js += u"var restricted_extent;\n"

    if area_name:
        js += u"var area_name='%s';\n" % area_name
    js = u"<script type='text/javascript'><!--\n"\
         u"%s// !--></script>\n" % js
    return js

def get_map_layers(area_name=''):
    from chimere.models import Area
    area = None
    if area_name:
        try:
            area = Area.objects.get(urn=area_name)
        except ObjectDoesNotExist:
            pass
    else:
        try:
            area = Area.objects.get(default=True)
        except ObjectDoesNotExist:
            pass
    map_layers, default = [], None
    if area and area.layers.count():
        map_layers = [layer.layer_code
                for layer in area.layers.order_by('arealayers__order').all()]
        def_layer = area.layers.filter(arealayers__default=True)
        if def_layer.count():
            def_layer = def_layer.all()[0]
            for order, map_layer in enumerate(map_layers):
                if map_layer == def_layer.layer_code:
                    default = order
    elif settings.CHIMERE_DEFAULT_MAP_LAYER:
        map_layers = [settings.CHIMERE_DEFAULT_MAP_LAYER]
    else:
        map_layers = ["new OpenLayers.Layer.OSM.Mapnik('Mapnik')"]
    return map_layers, default

class ChosenSelectWidget(forms.Select):
    """
    Chosen select widget.
    """
    class Media:
        js = ["%schosen/chosen/chosen.jquery.min.js" % settings.STATIC_URL,]
        css = {'all':
             ["%schosen/chosen/chosen.css" % settings.STATIC_URL,]
              }
    def render(self, *args, **kwargs):
        if 'attrs' not in kwargs:
            kwargs['attrs'] = {}
        kwargs['attrs'].update({'class': 'chzn-select'})
        rendered = super(ChosenSelectWidget, self).render(*args, **kwargs)
        rendered += u"\n<script type='text/javascript'>\n"\
        u"  $('#%s').chosen();\n"\
        u"</script>\n" % kwargs['attrs']['id']
        return mark_safe(rendered)

class ImporterChoicesWidget(forms.Select):
    '''
    Importer select widget.
    '''
    class Media:
        js = ["%schimere/js/importer_interface.js" % settings.STATIC_URL,]

class TextareaWidgetBase(forms.Textarea):
    """
    Manage the edition of a text using TinyMCE
    """
    def render(self, *args, **kwargs):
        if 'attrs' not in kwargs:
            kwargs['attrs'] = {}
        if 'class' not in kwargs['attrs']:
            kwargs['attrs']['class'] = ''
        else:
            kwargs['attrs']['class'] += ' '
        kwargs['attrs']['class'] += 'mceEditor'
        rendered = super(TextareaWidgetBase, self).render(*args, **kwargs)
        return mark_safe(rendered)

class TextareaWidget(TextareaWidgetBase):
    """
    Manage the edition of a text using TinyMCE
    """
    class Media:
        js = ["%stiny_mce.js" % settings.TINYMCE_URL,
              "%schimere/js/textareas.js" % settings.STATIC_URL,]

class TextareaAdminWidget(TextareaWidgetBase):
    class Media:
        js = ["%stiny_mce.js" % settings.TINYMCE_URL,
              "%schimere/js/textareas_admin.js" % settings.STATIC_URL,]

class DatePickerWidget(forms.TextInput):
    """
    Manage the edition of dates.
    JQuery and Jquery-UI are already loaded by default so don't include
    them in Media files.
    """
    def render(self, *args, **kwargs):
        rendered = super(DatePickerWidget, self).render(*args, **kwargs)
        rendered += u"\n<script type='text/javascript'>\n"\
        u"  $(function() {$('#%s').datepicker({ dateFormat: 'yy-mm-dd' });});\n"\
        u"</script>\n" % kwargs['attrs']['id']
        return mark_safe(rendered)

class PointChooserWidget(forms.TextInput):
    """
    Manage the edition of point on a map
    """
    class Media:
        css = {
            "all": settings.OSM_CSS_URLS + \
                   ["%schimere/css/forms.css" % settings.STATIC_URL,]
        }
        js = settings.OSM_JS_URLS + list(settings.JQUERY_JS_URLS) + \
             ["%schimere/js/jquery.chimere.js" % settings.STATIC_URL]

    def render(self, name, value, attrs=None, area_name=''):
        '''
        Render a map and latitude, longitude information field
        '''
        val = '0'
        value_x, value_y = 0, 0
        if value:
            val = str(value)
            if hasattr(value, 'x') and hasattr(value, 'y'):
                value_x, value_y = value.x, value.y
            elif isinstance(value, unicode) and value.startswith('POINT('):
                try:
                    value_x, value_y = value.split('(')[1][:-1].split(' ')
                    value_x, value_y = float(value_x), float(value_y)
                except:
                    value = None
            else:
                value = None
        map_layers, default_area = get_map_layers(area_name)
        js = """
        OpenLayers.ImgPath = '%schimere/img/';
        var EPSG_DISPLAY_PROJECTION = epsg_display_projection = new OpenLayers.Projection('EPSG:%s');
        var EPSG_PROJECTION = epsg_projection = new OpenLayers.Projection('EPSG:%s');
        var CENTER_LONLAT = centerLonLat = new OpenLayers.LonLat%s.transform(epsg_display_projection, epsg_projection);
        var DEFAULT_ZOOM = %s;
        var chimere_init_options = {};
        chimere_init_options["default_icon"] = new OpenLayers.Icon('%schimere/img/marker-green.png', new OpenLayers.Size(21, 25), new OpenLayers.Pixel(-(21/2), -25));
        chimere_init_options["map_layers"] = [%s];
        chimere_init_options['dynamic_categories'] = false;
        chimere_init_options['edition'] = true;
        chimere_init_options["checked_categories"] = [];
        """ % (settings.STATIC_URL, settings.CHIMERE_EPSG_DISPLAY_PROJECTION,
              settings.CHIMERE_EPSG_PROJECTION, settings.CHIMERE_DEFAULT_CENTER,
              settings.CHIMERE_DEFAULT_ZOOM, settings.STATIC_URL,
              ", ".join(map_layers))
        if default_area:
            js += "chimere_init_options['selected_map_layer'] = %d;\n" % \
                  default_area
        #TODO: manage area
        tpl = u"<script type='text/javascript'><!--\n"\
              u"%s// !--></script>\n" % js
        tpl += u"</div><div id='map_edit'></div>"\
               u"<div id='live_lonlat'>"\
               u"    <p><label for='live_latitude'>%s</label>"\
               u" <input type='texte' name='live_latitude' id='live_latitude' "\
               u"size='8' disabled='true' value='%f'/></p>"\
               u"<p><label for='live_longitude'>%s</label><input type='texte' "\
               u"name='live_longitude' id='live_longitude' size='8' "\
               u"disabled='true' value='%f'/></p>"\
               u"<input type='hidden' name='%s' id='id_%s' value='%s'/>" % (
                    _("Latitude"), value_y, _("Longitude"), value_x, name, name,
                    val)
        tpl += "<script type='text/javascript'><!--\n"
        tpl += "$('#map_edit').chimere(chimere_init_options);\n"
        if value:
            tpl += u'var mylonlat = new OpenLayers.LonLat(%f,%f);\n'\
                   u'$("#map_edit").chimere("putEditMarker", \n'\
                   u'   mylonlat.transform(EPSG_DISPLAY_PROJECTION, \n'\
                   u'                      EPSG_PROJECTION), true);\n' % (
                                                             value_x, value_y)
        tpl += "// --></script>\n"
        return mark_safe(tpl)

class PointField(models.PointField):
    '''
    Set the widget for the form field
    '''
    def formfield(self, **keys):
        defaults = {'widget': PointChooserWidget}
        keys.update(defaults)
        return super(PointField, self).formfield(**keys)

    def clean(self, value, instance=None):
        if len(value) != 2 and self.required:
            raise ValidationError(_("Invalid point"))
        return value

class RouteChooserWidget(forms.TextInput):
    """
    Manage the edition of route on a map
    """
    class Media:
        css = {"all": settings.OSM_CSS_URLS + \
                      ["%schimere/css/forms.css" % settings.STATIC_URL,]
        }
        js = settings.OSM_JS_URLS + list(settings.JQUERY_JS_URLS) + \
             ["%schimere/js/jquery.chimere.js" % settings.STATIC_URL,
              "%schimere/js/edit_route_map.js" % settings.STATIC_URL,
              "%schimere/js/base.js" % settings.STATIC_URL,]

    def render(self, name, value, attrs=None, area_name='', routefile_id=None):
        '''
        Render a map and latitude, longitude information field
        '''
        tpl = getMapJS(area_name)
        map_layers, default_area = get_map_layers(area_name)
        js = """
        var extra_url = "%s";
        OpenLayers.ImgPath = '%schimere/img/';
        var EPSG_DISPLAY_PROJECTION = epsg_display_projection = new OpenLayers.Projection('EPSG:%s');
        var EPSG_PROJECTION = epsg_projection = new OpenLayers.Projection('EPSG:%s');
        var CENTER_LONLAT = centerLonLat = new OpenLayers.LonLat%s.transform(epsg_display_projection, epsg_projection);
        var DEFAULT_ZOOM = %s;
        var chimere_init_options = {};
        chimere_init_options["map_layers"] = [%s];
        chimere_init_options['dynamic_categories'] = false;
        chimere_init_options['edition'] = true;
        chimere_init_options['edition_type_is_route'] = true;
        chimere_init_options["checked_categories"] = [];
        """ % ( reverse("chimere:index"), settings.STATIC_URL,
              settings.CHIMERE_EPSG_DISPLAY_PROJECTION,
              settings.CHIMERE_EPSG_PROJECTION, settings.CHIMERE_DEFAULT_CENTER,
              settings.CHIMERE_DEFAULT_ZOOM, ", ".join(map_layers))
        if default_area:
            js += "chimere_init_options['selected_map_layer'] = %d;\n" % \
                  default_area
        tpl = u"<script type='text/javascript'><!--\n"\
              u"%s// !--></script>\n" % js
        #TODO: manage area
        help_create = ''
        if not value:
            help_create = u"<h3>%s</h3>\n"\
            u"<p>%s</p>\n"\
            u"<p>%s</p>\n"\
            u"<p>%s</p>\n"\
            u"<p>%s</p>\n"\
            u"<p>%s</p>\n" % (_(u"Creation mode"),
            _(u"To start drawing the route click on the toggle button: "\
              u"\"Draw\"."),
            _(u"Then click on the map to begin the drawing."),
            _(u"You can add points by clicking again."),
            _(u"To finish the drawing double click. When the drawing is "\
              u"finished you can edit it."),
            _(u"While creating to undo a drawing click again on the toggle "\
              u"button \"Stop drawing\"."))
        help_modify = u"<h3>%s</h3>\n"\
        u"<p>%s</p>\n"\
        u"<p>%s</p>\n"\
        u"<p>%s</p>\n" % (_(u"Modification mode"),
        _(u"To move a point click on it and drag it to the desired position."),
        _(u"To delete a point move the mouse cursor over it and press the "\
          u"\"d\" or \"Del\" key."),
        _(u"To add a point click in the middle of a segment and drag the new "\
          u"point to the desired position"))
        if not value:
            # upload a file
            tpl += u"<script type='text/javascript'><!--\n"\
                   u"    var error_msg = \"%s\";"\
                   u"// --></script>" % (
                   _(u"Give a name and set category before uploading a file."))
            tpl += u'<div id="upload"><a href="#" class="upload-button" '\
               u'onclick="uploadFile(error_msg);return false;">%s</a></div>' % (
                    _(u"Upload a route file (GPX or KML)"))
            tpl += u"\n<p id='draw-or'>%s</p>\n" % _(u"or")
            tpl += u"<div id='draw-label'><div id='draw-toggle-off' "\
                   u"onclick='toggleDraw();'>\n"\
                   u"<a href='#' onclick='return false;'>%s</a></div>"\
                   u"</div>\n"\
                   u"<hr class='spacer'/>" % (_(u"Start \"hand\" drawing"))
        style = ''
        if value:
            style = " style='display:block'"
        tpl += u"\n<div class='help-route' id='help-route-modify'%s>%s</div>"\
        u"\n<hr class='spacer'/>\n"\
        u"<input type='hidden' name='%s' id='id_%s' value=\"%s\"/>\n"\
        u"<input type='hidden' name='associated_file_id' "\
        u"id='id_associated_file_id' value=\"%s\"/>\n" % (
            style, help_modify, name, name, value, routefile_id)
        if value:
            tpl += u"\n<div id='map_edit'></div>"
        else:
            tpl += "\n<div class='help-route' id='help-route-create'>%s</div>"\
                   % help_create
            tpl += u"\n<div id='map_edit'>\n"\
                   u"  <div class='map_button'>\n"\
            u"  <a href='#' id='button-move-map' class='toggle-button "\
            u"toggle-button-active' onclick='toggleDrawOff();return false;'>%s"\
            u"</a>\n"\
            u"<a href='#' id='button-draw-map' class='toggle-button "\
            u"toggle-button-inactive' onclick='toggleDrawOn();return false;'>"\
            u"%s</a></div>\n"\
            u"  </div>" % (_(u"Move on the map"), _(u"Draw"))
        tpl += u"<script type='text/javascript'><!--\n"
        if not value:
            tpl += u"jQuery('#map_edit').hide();"
        if value:
            tpl += u"jQuery('#map_edit').chimere(chimere_init_options);\n"
            val = value
            if type(value) == unicode:
                try:
                    val = fromstr(value)
                except:
                    pass
            if hasattr(val, 'json'):
                tpl += u"\nvar geometry='%s';\n"\
                       u"jQuery('#map_edit').chimere('initFeature', geometry);"\
                       % val.json
        tpl += u"\n// --></script>\n"
        return mark_safe(tpl)

class RouteField(models.LineStringField):
    '''
    Set the widget for the form field
    '''
    def formfield(self, **keys):
        defaults = {'widget': RouteChooserWidget}
        keys.update(defaults)
        return super(RouteField, self).formfield(**keys)

class AreaWidget(forms.TextInput):
    """
    Manage the edition of an area on the map
    """
    class Media:
        css = {
            "all": settings.OSM_CSS_URLS + \
                   ["%schimere/css/forms.css" % settings.STATIC_URL,]
        }
        js = settings.OSM_JS_URLS + [
              "%schimere/js/edit_area.js" % settings.STATIC_URL,
              "%schimere/js/base.js" % settings.STATIC_URL,]

    def get_bounding_box_from_value(self, value):
        '''
        Return upper left lat/lon and lower lat/lon from the input value
        '''
        upper_left_lat, upper_left_lon = 0, 0
        lower_right_lat, lower_right_lon = 0, 0
        if not value:
            return upper_left_lat, upper_left_lon, lower_right_lat, \
                   lower_right_lon
        if len(value) == 2:
            upper_left = value[0]
            lower_right = value[1]
            if hasattr(upper_left, 'x') and hasattr(upper_left, 'y'):
                upper_left_lon, upper_left_lat = upper_left.x, upper_left.y
            elif len(upper_left) == 2:
                try:
                    upper_left_lon = float(upper_left[0])
                    upper_left_lat = float(upper_left[1])
                except ValueError:
                    pass
            if hasattr(lower_right, 'x') and hasattr(lower_right, 'y'):
                lower_right_lon, lower_right_lat = lower_right.x, \
                                                   lower_right.y
            elif len(lower_right) == 2:
                lower_right_lon, lower_right_lat = lower_right
                try:
                    lower_right_lon = float(lower_right[0])
                    lower_right_lat = float(lower_right[1])
                except ValueError:
                    pass
        return upper_left_lat, upper_left_lon, lower_right_lat, lower_right_lon

    def render(self, name, value, attrs=None, initialized=True):
        """
        Render a map
        """
        upper_left_lat, upper_left_lon, lower_right_lat, lower_right_lon = \
                                        self.get_bounding_box_from_value(value)
        tpl = getMapJS()
        tpl += u"</div>\n"\
        u"<input type='hidden' name='upper_left_lat' id='upper_left_lat' "\
        u"value='%f'/>\n"\
        u"<input type='hidden' name='upper_left_lon' id='upper_left_lon' "\
        u"value='%f'/>\n"\
        u"<input type='hidden' name='lower_right_lat' id='lower_right_lat' "\
        u"value='%f'/>\n"\
        u"<input type='hidden' name='lower_right_lon' id='lower_right_lon' "\
        u"value='%f'/>\n" % (
            upper_left_lat, upper_left_lon, lower_right_lat, lower_right_lon)
        help_msg = _(u"Hold CTRL, click and drag to select area on the map")
        tpl += u"<p class='help-osm'>%s</p>\n" % help_msg
        tpl += u"<script type='text/javascript'>\n"
        tpl += u"function init_map_form (){\ninit();\n"
        if value:
            tpl += u"var extent = new OpenLayers.Bounds(%f, %f, %f, %f);\n"\
            u"extent.transform(epsg_display_projection, epsg_projection);\n"\
            u"updateForm(extent);\n"\
            u"map.zoomToExtent(extent, true);\n"\
            u"map.zoomOut();" % (upper_left_lon, upper_left_lat,
                                 lower_right_lon, lower_right_lat)
        tpl += u"}\n"
        if initialized:
            tpl += u"$(document).ready(function($) {init_map_form()});\n"
        tpl += u"</script>\n"
        tpl += u"<div id='map_edit'>\n"
        return mark_safe(tpl)

    def value_from_datadict(self, data, files, name):
        """
        Return the appropriate values
        """
        values = []
        for keys in (('upper_left_lon', 'upper_left_lat',),
                     ('lower_right_lon', 'lower_right_lat')):
            value = []
            for key in keys:
                val = data.get(key, None)
                if not val:
                    return []
                value.append(val)
            values.append(value)
        return values

RE_XAPI = re.compile('(node|way)\[(.*=.*)\]\[bbox='\
          '(-*[0-9]*.[0-9]*,-*[0-9]*.[0-9]*,-*[0-9]*.[0-9]*,-*[0-9]*.[0-9]*)\]')

class ImportFiltrWidget(AreaWidget):
    """
    Manage the edition of the import source field
    """
    class Media:
        css = {
            "all": settings.OSM_CSS_URLS + \
                   ["%schimere/css/forms.css" % settings.STATIC_URL,]
        }
        js = settings.OSM_JS_URLS + [
              "%schimere/js/edit_area.js" % settings.STATIC_URL,
              "%schimere/js/base.js" % settings.STATIC_URL,]

    def render(self, name, value, attrs=None):
        """
        Render a map
        """
        tpl = super(ImportFiltrWidget, self).render(name, value, attrs,
                                                     initialized=False)
        tpl += u"</div><hr class='spacer'/>"
        vals = {'lbl':_(u"Type:"), 'name':name, 'node':_(u"Node"),
                'way':_(u"Way")}
        vals['way_selected'] = ' checked="checked"'\
                               if self.xapi_type == 'way' else ''
        vals['node_selected'] = ' checked="checked"'\
                               if self.xapi_type == 'node' else ''
        tpl += u"<div class='input-osm'><label>%(lbl)s</label>"\
         u"<input type='radio' name='id_%(name)s_type' id='id_%(name)s_node'"\
         u" value='node'%(node_selected)s/> <label for='id_%(name)s_node'>"\
         u"%(node)s</label> <input type='radio' name='id_%(name)s_type' "\
         u"id='id_%(name)s_way' value='way'%(way_selected)s/> <label "\
         u"for='id_%(name)s_way'>%(way)s</label></div>" % vals
        help_msg = _(u"Enter an OSM \"tag=value\" string such as "\
             u"\"amenity=pub\". A list of common tag is available "\
             u"<a href='https://wiki.openstreetmap.org/wiki/Map_Features' "\
             u" target='_blank'>here</a>.")
        tpl += u"<p class='help-osm'>%s</p>\n" % help_msg
        tpl += u"<div class='input-osm'><label for='id_%s_tag'>%s</label>"\
               u"<input type='text' id='id_%s_tag' value=\"%s\"/></div>" % (
                                         name, _(u"Tag:"), name, self.xapi_tag)
        tpl += u"<script type='text/javascript'>\n"
        tpl += u"var default_xapi='%s';" % settings.CHIMERE_XAPI_URL
        tpl += u'var msg_missing_area = "%s";' % \
                                            _(u"You have to select an area.")
        tpl += u'var msg_missing_type = "%s";' % \
                                            _(u"You have to select a type.")
        tpl += u'var msg_missing_filtr = "%s";' % \
                                         _(u"You have to insert a filter tag.")
        tpl += u"</script>\n"
        help_msg = _(u"If you change the above form don't forget to refresh "\
                     u"before submit!")
        tpl += u"<p class='help-osm errornote'>%s</p>\n" % help_msg
        help_msg = _(u"You can put a Folder name of the KML file to filter on "\
                     u"it.")
        tpl += u"<p class='help-kml'>%s</p>\n" % help_msg
        if not value:
            value = ''
        tpl += u"<div><input type='text' id='id_%s' name='id_%s' "\
               u"value=\"%s\"/> <input type='button' id='id_refresh_%s' "\
               u"value='%s' class='input-osm'/>" % (name, name, value, name,
                                                    _(u"Refresh"))
        return mark_safe(tpl)

    def value_from_datadict(self, data, files, name):
        """
        Return the appropriate values
        """
        return data.get('id_'+name, None)

    def get_bounding_box_from_value(self, value):
        '''
        Return upper left lat/lon, lower lat/lon from the input value.
        Get also xapi type and xapi tag
        '''
        upper_left_lat, upper_left_lon = 0, 0
        lower_right_lat, lower_right_lon = 0, 0
        self.xapi_type, self.xapi_tag, self.bounding_box = None, '', None
        if not value:
            return upper_left_lat, upper_left_lon, lower_right_lat, \
                   lower_right_lon
        xapi_m = RE_XAPI.match(value)
        if not xapi_m:
            return upper_left_lat, upper_left_lon, lower_right_lat, \
                   lower_right_lon
        # as the regexp pass, we could be pretty confident
        self.xapi_type, self.xapi_tag, self.bounding_box = xapi_m.groups()
        upper_left_lon, lower_right_lat, lower_right_lon, upper_left_lat = \
                self.bounding_box.split(',')
        return float(upper_left_lat), float(upper_left_lon), \
               float(lower_right_lat), float(lower_right_lon)

class AreaField(forms.MultiValueField):
    '''
    Set the widget for the form field
    '''
    widget = AreaWidget

    def compress(self, data_list):
        if not data_list:
            return None
        return data_list

class MultiSelectWidget(forms.SelectMultiple):
    class Media:
        css = {'all': list(settings.JQUERY_CSS_URLS) + [
            settings.STATIC_URL + 'bsmSelect/css/jquery.bsmselect.css',
            settings.STATIC_URL + 'bsmSelect/css/jquery.bsmselect.custom.css',
            ]
        }
        js = list(settings.JQUERY_JS_URLS) + [
            settings.STATIC_URL + 'bsmSelect/js/jquery.bsmselect.js',
            settings.STATIC_URL + 'bsmSelect/js/jquery.bsmselect.compatibility.js',
            ]

    def render(self, name, value, attrs=None):
        rendered = super(MultiSelectWidget, self).render(name, value, attrs)
        rendered += u"<hr class='spacer'/><script type='text/javascript'>\n"\
        u"$.bsmSelect.conf['title'] = \"%(title)s\";\n"\
        u"$(\"#id_%(name)s\").bsmSelect({\n"\
        u"    removeLabel: '<strong>X</strong>',\n"\
        u"    containerClass: 'bsmContainer',\n"\
        u"    listClass: 'bsmList-custom',\n"\
        u"    listItemClass: 'bsmListItem-custom',\n"\
        u"    listItemLabelClass: 'bsmListItemLabel-custom',\n"\
        u"    removeClass: 'bsmListItemRemove-custom'\n"\
        u"});\n"\
        u"</script>\n" % {'name':name, 'title':_("Select...")}
        return mark_safe(rendered)

class SelectMultipleField(models.ManyToManyField):
    '''
    Set the widget for the category field
    '''
    def formfield(self, **keys):
        self.help_text = ""
        defaults = {'widget': MultiSelectWidget}
        keys.update(defaults)
        return super(SelectMultipleField, self).formfield(**keys)

from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^chimere\.widgets\.PointField"])
add_introspection_rules([], ["^chimere\.widgets\.SelectMultipleField"])
add_introspection_rules([], ["^chimere\.widgets\.RouteField"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import csv
import datetime, time

from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ObjectDoesNotExist

from chimere.models import Importer
from chimere import tasks

class Command(BaseCommand):
    args = '<import_id>'
    help = "Launch import from an external source. Import configuration must "\
           "be beforehand inserted in the database with the web admin."

    def handle(self, *args, **options):
        imports = dict([(imp.pk, imp)
                       for imp in Importer.objects.order_by('pk').all()])
        imprt = None
        if args and args[0]:
            try:
                imprt = Importer.objects.get(pk=int(args[0]))
            except (ValueError, ObjectDoesNotExist):
                raise CommandError("Import with ID '%s' doesn't exist." % \
                                                                        args[0])
        if not args:
            while not imprt:
                self.stdout.write('* Choose the import: \n')
                for k in imports:
                    self.stdout.write(' %s\n' % unicode(imports[k]).encode(
                                                             'ascii', 'ignore'))
                self.stdout.flush()
                self.stdout.write('\nImport ID: ')
                v = raw_input()
                try:
                    imprt = Importer.objects.get(pk=int(v))
                except (ValueError, ObjectDoesNotExist):
                    self.stdout.write("Import with ID '%s' doesn't exist.\n\n" \
                                      % v)
        pending_state = unicode(tasks.IMPORT_MESSAGES['import_pending'][0])
        imprt.state = pending_state
        imprt.save()
        tasks.importing(imprt.pk)
        self.stdout.write("Import '%d' in progress...\n" % imprt.pk)
        self.stdout.flush()
        state = pending_state
        while state == pending_state:
            time.sleep(1)
            state = Importer.objects.get(pk=int(imprt.pk)).state
        self.stdout.write(state + '\n')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Utilitaries
"""

import csv
import datetime
import feedparser
import os
import re
import StringIO
import tempfile
import urllib2
import unicodedata
import zipfile

from osgeo import ogr, osr
from lxml import etree

from django.conf import settings
from django.contrib.gis.gdal import DataSource, OGRGeomType, check_err
from django.contrib.gis.geos import GEOSGeometry
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import render_to_response
from django.utils.translation import ugettext_lazy as _

from chimere import get_version
from external_utils import OsmApi

def unicode_normalize(string):
    if type(string) == str:
        string = unicode(string.decode('utf-8'))
    return ''.join(
        (c for c in unicodedata.normalize('NFD', string)
        if unicodedata.category(c) != 'Mn'))

class ImportManager(object):
    u"""
    Generic class for specific importers
    """
    default_source = None
    def __init__(self, importer_instance):
        self.importer_instance = importer_instance
        if self.importer_instance.default_name:
            self.default_name = self.importer_instance.default_name
        else:
            self.default_name = " - ".join([cat.name
                for cat in self.importer_instance.categories.order_by(
                                                               'name').all()])

    def get(self):
        pass

    def put(self, extra_args={}):
        pass

    def create_or_update_item(self, cls, values, import_key, version=None,
                              key='', pk=None):
        updated, created, item = False, False, None
        import_key = unicode(import_key).replace(':', '^')
        if not values.get('name'):
            values['name'] = self.default_name
        if not key:
            key = self.importer_instance.importer_type
        item = None
        if import_key or pk:
            dct_import = {
                'import_key__icontains':'%s:%s;' % (key, import_key),
                'import_source':self.importer_instance.source}
            try:
                if pk:
                    ref_item = cls.objects.get(pk=pk)
                else:
                    ref_item = cls.objects.filter(**dct_import)
                    if not ref_item.count():
                        raise ObjectDoesNotExist
                    ref_item = ref_item.all()[0]
                if version and ref_item.import_version == int(version):
                    # no update since the last import
                    return ref_item, None, None
                if not self.importer_instance.overwrite \
                   and ref_item.modified_since_import:
                    dct_import['ref_item'] = ref_item
                else:
                    item = ref_item
                    for k in values:
                        if values[k]:
                            setattr(item, k, values[k])
                    try:
                        item.save()
                        # force the modified_since_import status
                        item.modified_since_import = False
                        item.save()
                    except TypeError:
                        # error on data source
                        return None, False, False
                    updated = True
            except ObjectDoesNotExist:
                pass
        if not item:
            if not self.importer_instance.get_description and \
               self.importer_instance.default_description:
                values['description'] = \
                                     self.importer_instance.default_description
            values.update({
                'import_source':self.importer_instance.source})
            values['status'] = 'I'
            if not self.importer_instance.associate_marker_to_way\
              and cls.__name__ == 'Route':
                values['has_associated_marker'] = False
            try:
                item = cls.objects.create(**values)
            except TypeError:
                # error on data source
                return None, False, False
            created = True
        if import_key:
            item.set_key(key, import_key)
        item.categories.clear()
        for cat in self.importer_instance.categories.all():
            item.categories.add(cat)
        return item, updated, created

    @classmethod
    def get_files_inside_zip(cls, zippedfile, suffixes, dest_dir=None):
        try:
            flz = zipfile.ZipFile(zippedfile)
        except zipfile.BadZipfile:
            return [], _(u"Bad zip file")
        namelist = flz.namelist()
        filenames = []
        for suffix in suffixes:
            current_file_name = None
            for name in namelist:
                if name.endswith(suffix) \
                  or name.endswith(suffix.lower()) \
                  or name.endswith(suffix.upper()):
                    current_file_name = name
            filenames.append(current_file_name)
        files = []
        for filename in filenames:
            if filename:
                if dest_dir:
                    files.append(filename)
                    flz.extract(filename, dest_dir)
                else:
                    files.append(flz.open(filename))
            else:
                files.append(None)
        return files

    def get_source_file(self, suffixes, dest_dir=None,
                        extra_url=None):
        source = self.importer_instance.source_file
        if not hasattr(source, 'read'):
            if not source:
                source = self.importer_instance.source \
                       if self.importer_instance.source else self.default_source
            try:
                url = source
                if extra_url:
                    url += extra_url
                remotehandle = urllib2.urlopen(url)
                source = StringIO.StringIO(remotehandle.read())
                remotehandle.close()
            except ValueError:
                # assume it is a local file
                try:
                    source = open(source)
                except IOError, msg:
                    return (None, msg)
            except (urllib2.URLError, AttributeError) as error:
                return (None, error.message)
        if self.importer_instance.zipped:
            try:
                files = self.get_files_inside_zip(source, suffixes, dest_dir)
            except zipfile.BadZipfile:
                return (None, _(u"Bad zip file"))
            if not files or None in files:
                return (None,
                        _(u"Missing file(s) inside the zip file"))
            source = files[0] if len(suffixes) == 1 else files
        return (source, None)

class KMLManager(ImportManager):
    u"""
    KML importer
    The filtr argument has to be defined as the exact name of the folder to be
    imported
    """
    XPATH = '//kml:Folder/kml:name[text()="%s"]/../kml:Placemark'
    DEFAULT_XPATH = '//kml:Placemark'
    def __init__(self, importer_instance, ns=''):
        super(KMLManager, self).__init__(importer_instance)
        self.ns = ns

    def get(self):
        u"""
        Get data from a KML source

        Return a tuple with:
         - number of new item ;
         - number of item updated ;
         - error detail on error
        """
        from models import Marker, Route
        new_item, updated_item, msg = 0, 0, ''
        source, msg = self.get_source_file(['.kml'])
        if msg:
            return (0, 0, msg)
        doc = source
        # remove empty lines before declaration (bad XML file)
        if hasattr(source, 'getvalue'):
            splitted = source.getvalue().split('\n')
            for idx, line in enumerate(splitted):
                if line.strip():
                    break
            doc = StringIO.StringIO("\n".join(splitted[idx:]))
        try:
            tree = etree.parse(doc)
        except:
            return (0, 0, _(u"Bad XML file"))
        # try to get default namespace
        if not self.ns:
            self.ns = tree.getroot().nsmap[None]
        xpath = self.XPATH % self.importer_instance.filtr \
                  if self.importer_instance.filtr else self.DEFAULT_XPATH
        for placemark in tree.xpath(xpath,
                                    namespaces={'kml':self.ns}):
            name, point, line = None, None, None
            pl_id = placemark.attrib.get('id')
            pl_key = 'kml-%d' % self.importer_instance.pk
            ns = '{%s}' % self.ns
            description = ''
            for item in placemark:
                if item.tag == ns + 'name':
                    name = item.text
                    if not pl_id:
                        # if no ID is provided assume that name is a key
                        pl_id = name
                elif item.tag == ns + 'description':
                    if self.importer_instance.get_description:
                        description = item.text
                elif item.tag == ns + 'Point':
                    for coord in item:
                        if coord.tag == ns + 'coordinates':
                            x, y, z = coord.text.split(',')
                            point = 'SRID=4326;POINT(%s %s)' % (x, y)
                elif item.tag == ns + 'LineString':
                    for coord in item:
                        if coord.tag == ns + 'coordinates':
                            points = coord.text.replace('\n', ' ').split(' ')
                            points = ",".join([" ".join(p.split(',')[:2])
                                               for p in points if p])
                            line = 'SRID=4326;LINESTRING(%s)' % points
            cls = None
            dct = {'description':description,
                   'name':name,
                   'origin':self.importer_instance.origin,
                   'license':self.importer_instance.license}
            if point:
                dct['point'] = point
                cls = Marker
            if line:
                dct['route'] = line
                dct.pop('description')
                cls = Route
            if cls:
                item, updated, created = self.create_or_update_item(
                                                cls, dct, pl_id, key=pl_key)
                if updated:
                    updated_item += 1
                if created:
                    new_item += 1
        return (new_item, updated_item, msg)

    @classmethod
    def export(cls, queryset):
        dct = {'name':settings.PROJECT_NAME,
               'description':unicode(datetime.date.today()),
               'locations':queryset.all()
        }
        filename = unicode_normalize(settings.PROJECT_NAME + dct['description']\
                                     + '.kml')
        result = render_to_response('chimere/export.kml', dct)
        return filename, result

class ShapefileManager(ImportManager):
    u"""
    Shapefile importer
    """
    def get(self):
        u"""
        Get data from a Shapefile source

        Return a tuple with:
         - number of new item ;
         - number of item updated ;
         - error detail on error
        """
        from models import Marker, Route
        new_item, updated_item, msg = 0, 0, ''
        tmpdir = tempfile.mkdtemp()
        sources, msg = self.get_source_file(['.shp', '.dbf', '.prj', '.shx'],
                                            dest_dir=tmpdir)
        if msg:
            return (0, 0, msg)
        if not sources:
            return (0, 0, _(u"Error while reading the data source."))
        # get the srid
        srid = self.importer_instance.srid
        if not srid:
            prjfilename = tmpdir + os.sep + sources[2]
            try:
                from osgeo import osr
                with open(prjfilename, 'r') as prj_file:
                    prj_txt = prj_file.read()
                    srs = osr.SpatialReference()
                    srs.ImportFromESRI([prj_txt])
                    srs.AutoIdentifyEPSG()
                    srid = srs.GetAuthorityCode(None)
            except ImportError:
                pass
            if not srid:
                # try with the default projection
                srid = settings.CHIMERE_EPSG_DISPLAY_PROJECTION
                msg = _(u"SRID cannot be guessed. The default SRID (%s) has "
                        u"been used.") % srid
                #If imported items are not well located "
                #        u"ask your data provider for the SRID to use.") % srid
        shapefilename = tmpdir + os.sep + sources[0]
        ds = DataSource(shapefilename)
        lyr = ds[0]
        # for this first version it is assumed that the first field is a
        # id name and the second field is the name
        id_name = lyr.fields[0] if len(lyr.fields) > 0 else None
        # test if id_name is well guess
        if id_name:
            ids = lyr.get_fields(id_name)
            if len(ids) != len(set(ids)):
                id_name = None
        lbl_name = None
        if len(lyr.fields) > 1:
            lbl_name = lyr.fields[1]
        elif id_name:
            lbl_name = id_name
        if lyr.geom_type not in ('Point', 'LineString'):
            return (0, 0, _(u"Type of geographic item of this shapefile "
                            u"is not managed by Chimère."))
        geom_key = 'point' if lyr.geom_type == 'Point' else 'route'
        geom_cls = Marker if lyr.geom_type == 'Point' else Route
        indexes = []
        for idx, feat in enumerate(lyr):
            name = unicode(idx)
            if lbl_name:
                name = feat.get(lbl_name)
                try:
                    name = unicode(name)
                except UnicodeDecodeError:
                    try:
                        name = unicode(
                               name.decode(settings.CHIMERE_SHAPEFILE_ENCODING))
                    except:
                        continue
            try:
                geoms = [feat.geom.wkt]
            except:
                return (0, 0, _(u"Bad Shapefile"))
            if feat.geom.geom_type == 'MultiLineString':
                geoms = [geom.wkt for geom in feat.geom]
            import_key = feat.get(id_name) if id_name and len(geoms) == 1 else ''
            for geom in geoms:
                dct = {geom_key:'SRID=%s;%s' % (srid, geom),
                   'name':name,
                   'origin':self.importer_instance.origin,
                   'license':self.importer_instance.license
                  }
                item, updated, created = self.create_or_update_item(
                                    geom_cls, dct, import_key)
                if updated:
                    updated_item += 1
                if created:
                    new_item += 1
        # clean up
        tmpdirs = set()
        for src in sources:
            dirs = os.sep.join(src.split(os.sep)[:-1])
            if dirs:
                tmpdirs.add(tmpdir + os.sep + dirs)
            os.remove(tmpdir + os.sep + src)
        for dr in tmpdirs:
            os.removedirs(dr)
        return (new_item, updated_item, msg)

    @classmethod
    def export(cls, queryset):
        date = unicode(datetime.date.today())

        tmp = tempfile.NamedTemporaryFile(suffix='.shp', mode='w+b')
        tmp.close()

        tmp_name = tmp.name
        field_names = [field.name for field in queryset.model._meta.fields]
        geo_field = getattr(queryset.model,
                          'point' if 'point' in field_names else 'route')._field

        dr = ogr.GetDriverByName('ESRI Shapefile')
        ds = dr.CreateDataSource(tmp_name)
        if ds is None:
            raise Exception(_(u'Could not create file!'))
        ogr_type = OGRGeomType(geo_field.geom_type).num
        srs = osr.SpatialReference()
        srs.ImportFromEPSG(geo_field.srid)

        layer = ds.CreateLayer('lyr', srs=srs, geom_type=ogr_type)

        for field_name in ('name', 'category'):
            field_defn = ogr.FieldDefn(str(field_name), ogr.OFTString)
            field_defn.SetWidth(255)
            if layer.CreateField(field_defn) != 0:
                raise Exception(_(u'Failed to create field'))

        feature_def = layer.GetLayerDefn()

        for item in queryset:
            # duplicate items when in several categories
            for category in item.categories.all():
                feat = ogr.Feature(feature_def)
                feat.SetField('name', str(unicode_normalize(item.name)[:80]))
                feat.SetField('category',
                                   str(unicode_normalize(category.name)[:80]))

                geom = getattr(item, geo_field.name)
                if not geom:
                    continue
                ogr_geom = ogr.CreateGeometryFromWkt(geom.wkt)
                check_err(feat.SetGeometry(ogr_geom))
            check_err(layer.CreateFeature(feat))
        # Cleaning up
        ds.Destroy()

        # writing to a zip file
        filename = unicode_normalize(settings.PROJECT_NAME) + '-' + date
        buff = StringIO.StringIO()
        zip_file = zipfile.ZipFile(buff, 'w', zipfile.ZIP_DEFLATED)
        suffixes = ['shp', 'shx', 'prj', 'dbf']
        for suffix in suffixes:
            name = tmp_name.replace('.shp', '.' + suffix)
            arcname = '.'.join((filename, suffix))
            zip_file.write(name, arcname=arcname)
        zip_file.close()
        buff.flush()
        zip_stream = buff.getvalue()
        buff.close()
        return filename, zip_stream

class CSVManager(ImportManager):
    u"""
    CSV importer
    """
    @classmethod
    def set_categories(value):
        return

    # (label, getter, setter)
    COLS = [("Id", 'pk', 'pk'), (_(u"Name"), 'name', 'name'),
            (_(u"Categories"), lambda obj:", ".join(
                                [c.name for c in obj.categories.all()]),
                                set_categories),
            (_(u"State"), 'status', lambda x: x),
            (_(u"Description"), 'description', 'description'),
            (_(u"Localisation"), 'geometry', 'geometry')]

    def get(self):
        u"""
        Get data from a CSV source

        Return a tuple with:
         - number of new item ;
         - number of item updated ;
         - error detail on error
        """
        from models import Marker, Route
        new_item, updated_item, msg = 0, 0, ''
        source, msg = self.get_source_file(['.csv'])
        if msg:
            return (0, 0, msg)
        reader = csv.reader(source, delimiter=';', quotechar='"')
        prop_cols = []
        for pm in Marker.properties():
            prop_cols.append((pm.name, pm.getAttrName(),
                              pm.getAttrName()+'_set'))
        cols = self.COLS + prop_cols
        datas = []
        for idx, row in enumerate(reader):
            if not idx: # first row
                try:
                    assert(len(row) >= len(cols))
                except AssertionError:
                    return (0, 0, _(u"Invalid CSV format"))
                continue
            if len(row) < len(cols):
                continue
            pk, name, cats, state = row[0], row[1], row[2], row[3]
            geom = row[5]
            description = ''
            if self.importer_instance.get_description:
                description = row[4]
            COL_INDEX = 6
            dct = {'description':description,
                   'name':name,
                   'origin':self.importer_instance.origin,
                   'license':self.importer_instance.license}
            cls = None
            if 'POINT' in geom:
                cls = Marker
                dct['point'] = geom
            elif 'LINE' in geom:
                cls = Route
                dct['route'] = geom
            else:
                continue
            import_key = pk if pk else name
            item, updated, created = self.create_or_update_item(cls, dct,
                                                             import_key, pk=pk)
            if updated:
                updated_item += 1
            if created:
                new_item += 1
            for idx, col in enumerate(cols[COL_INDEX:]):
                name, getter, setter_val = col
                setter = getattr(item, setter_val)
                val = row[idx+COL_INDEX]
                setter(item, val)
        return (new_item, updated_item, msg)

    @classmethod
    def export(cls, queryset):
        dct = {'description':unicode(datetime.date.today()), 'data':[]}
        cls_name = queryset.model.__name__.lower()
        cols = cls.COLS
        for pm in queryset.model.properties():
            cols.append((pm.name, pm.getAttrName(), pm.getAttrName()+'_set'))
        header = [col[0] for col in cols]
        dct['data'].append(header)
        for item in queryset.all():
            data = []
            for (lbl, attr, setr) in cols:
                if callable(attr):
                    data.append(attr(item))
                else:
                    data.append(getattr(item, attr))
            dct['data'].append(data)
        filename = unicode_normalize(settings.PROJECT_NAME + dct['description']\
                                     + '.csv')
        result = render_to_response('chimere/export.csv', dct)
        return filename, result

class GeoRSSManager(ImportManager):
    u"""
    RSS importer.
    This manager only gets and do not produce GeoRSSFeed
    """

    def get(self):
        u"""
        Get data from a GeoRSS simple source

        Return a tuple with:
         - number of new item ;
         - number of item updated ;
         - error detail on error
        """
        from models import Marker
        new_item, updated_item, msg = 0, 0, ''
        feed = feedparser.parse(self.importer_instance.source)
        if feed['bozo']:
            return (0, 0, _(u"RSS feed is not well formed"))
        for item in feed['items']:
            if "georss_point" not in item and 'georss_line' not in item \
               and not ("geo_lat" in item and "geo_long" in item):
                continue
            cls = None
            dct = {'origin':self.importer_instance.origin,
                   'license':self.importer_instance.license}
            if 'georss_point' in item or "geo_lat" in item:
                cls = Marker
                if 'georss_point' in item:
                    try:
                        y, x = item['georss_point'].split(' ')
                    except ValueError:
                        continue
                else:
                    y = item['geo_lat']
                    x = item['geo_long']
                dct['point'] = 'SRID=4326;POINT(%s %s)' % (x, y)
                if self.importer_instance.get_description:
                    for k in ['description', 'summary', 'value']:
                        if k in item:
                            dct['description'] = item[k]
                            break
            else:
                cls = Route
                points = item['georss_line'].split(' ')
                reordered_points = []
                # lat, lon -> x, y
                for idx in xrange(len(points)/2):
                    reordered_points.append("%s %s" % (points[idx*2+1],
                                                       points[idx*2]))
                dct['route'] = 'SRID=4326;LINESTRING(%s)' % \
                            ",".join(reordered_points)

            dct['name'] = item['title']
            pl_id = item['id'] if 'id' in item else item['title']
            it, updated, created = self.create_or_update_item(cls, dct, pl_id)
            if updated:
                updated_item += 1
            if created:
                new_item += 1
        return (new_item, updated_item, msg)

RE_HOOK = re.compile('\[([^\]]*)\]')

# TODO: manage deleted item from OSM

class OSMManager(ImportManager):
    u"""
    OSM importer/exporter
    The source url is a path to an OSM file or a XAPI url
    The filtr argument is XAPI args or empty if it is an OSM file.
    """
    default_source = settings.CHIMERE_XAPI_URL

    def get(self):
        u"""
        Get data from the source

        Return a tuple with:
        - new items;
        - updated items;
        - error detail on error.
        """
        source, msg = self.get_source_file(['.osm'],
                                         extra_url=self.importer_instance.filtr)
        if not source:
            return (0, 0, msg)

        tree = etree.parse(source)
        # only import node or ways
        if tree.xpath('count(//way)') and tree.xpath('count(//node)'):
            return self.import_ways(tree)
        elif tree.xpath('count(//node)'):
            return self.import_nodes(tree)
        return 0, 0, _(u"Nothing to import")

    def import_ways(self, tree):
        from chimere.models import Marker, Route
        msg, items, new_item, updated_item = "", [], 0 , 0
        nodes = {}
        for node in tree.xpath('//node'):
            node_id = node.attrib.get('id')
            for item in node:
                k = item.attrib.get('k')
            if node_id:
                nodes[node_id] = '%s %s' % (node.get('lon'),
                                            node.get('lat'))
        for way in tree.xpath('//way'):
            name = None
            points = []
            node_id = way.attrib.get('id')
            version = way.attrib.get('version')
            for item in way:
                k = item.attrib.get('k')
                if k == 'name':
                    name = item.attrib.get('v')
                if item.tag == 'nd':
                    points.append(item.get('ref'))
            if not points:
                continue
            wkt = 'SRID=4326;LINESTRING(%s)' % ",".join([nodes[point_id]
                            for point_id in points if point_id in nodes])
            dct = {'route':wkt,
                   'name':name,
                   'origin':self.importer_instance.origin \
                            or u'OpenStreetMap.org',
                   'license':self.importer_instance.license \
                             or u'ODbL',
                   'import_version':version}
            item, updated, created = self.create_or_update_item(
                                    Route, dct, node_id, version)
            if updated:
                updated_item += 1
            if created:
                new_item += 1
            items.append(item)
        return new_item, updated_item, msg

    def import_nodes(self, tree):
        from chimere.models import Marker
        msg, items, new_item, updated_item = "", [], 0 , 0
        for node in tree.xpath('//node'):
            name = None
            node_id = node.attrib.get('id')
            if not node_id:
                continue
            version = node.attrib.get('version')
            for item in node:
                k = item.attrib.get('k')
                if k == 'name':
                    name = item.attrib.get('v')
            point = 'SRID=4326;POINT(%s %s)' % (node.get('lon'),
                                                node.get('lat'))
            dct = {'point':point,
                   'name':name,
                   'origin':self.importer_instance.origin \
                            or u'OpenStreetMap.org',
                   'license':self.importer_instance.license \
                             or u'ODbL',
                   'import_version':version}
            item, updated, created = self.create_or_update_item(
                                    Marker, dct, node_id, version)
            if updated:
                updated_item += 1
            if created:
                new_item += 1
            items.append(item)
        return (new_item, updated_item, msg)

    def put(self, extra_args={}):
        # first of all: reimport in order to verify that no changes has been
        # made since the last import
        from models import Marker
        new_item, updated_item, msg = self.get()
        # check if import is possible
        if msg:
            return 0, msg
        if new_item:
            return 0, _(u"New items imported - validate them before exporting")
        if Marker.objects.filter(status='I').count():
            return 0, _(u"There are items from a former import not yet "
                        u"validated - validate them before exporting")
        # start import
        api = settings.CHIMERE_OSM_API_URL
        username = settings.CHIMERE_OSM_USER
        password = settings.CHIMERE_OSM_PASSWORD
        if extra_args:
            try:
                api = extra_args['api']
                username = extra_args['username']
                password = extra_args['password']
            except KeyError:
                return 0, _(u"Bad params - programming error")
        username = username.encode('latin1')
        password = password.encode('latin1')
        api = OsmApi.OsmApi(api=api, username=username, password=password)
        api.ChangesetCreate({u"comment": u"Import from Chimère %s" % \
                                                            get_version()})
        hooks = RE_HOOK.findall(self.importer_instance.filtr)
        if not hooks:
            hooks = RE_HOOK.findall(self.importer_instance.source)
            if not hooks:
                return 0, _(u"Bad param")
        tags = {}
        bbox = []
        for hook in hooks:
            key, value = hook.split('=')
            if '*' in value or '|' in key or '|' in value:
                continue
            if key == 'bbox':
                x1, y1, x2, y2 = [float(val) for val in value.split(',')]
                bbox =  GEOSGeometry(
                    'POLYGON((%f %f,%f %f,%f %f,%f %f,%f %f))' % (
                    x1, y1, x2, y1, x2, y2, x1, y2, x1, y1), srid=4326)
                continue
            tags[key] = value
        if not tags:
            return 0, _(u"No non ambigious tag is defined in the XAPI request")
        if not bbox:
            return 0, _(u"No bounding box is defined in the XAPI request."\
            u"If you are sure to manage the entire planet set the bounding box"\
            u" to -180,-90,180,90")
        default_dct = {'tag':tags,
                       'import_source':self.importer_instance.source}
        idx = -1
        for idx, item in enumerate(Marker.objects.filter(status='A',
                point__contained=bbox,
                categories=self.importer_instance.categories.all(),
                not_for_osm=False, modified_since_import=True,
                route=None).all()):
            dct = default_dct.copy()
            dct.update({'lon':item.point.x,
                        'lat':item.point.y})
            dct['tag']['name'] = item.name
            node = None
            import_key = item.get_key('OSM')
            updated = False
            if import_key:
                try:
                    dct['id'] = import_key
                    dct['version'] = item.import_version
                    node = api.NodeUpdate(dct)
                    updated = True
                except OsmApi.ApiError, error:
                    if error.status == 404:
                        dct.pop('id')
                        dct.pop('version')
                        pass # if the node doesn't exist it is created
                    else:
                        raise
            if not updated:
                node = api.NodeCreate(dct)
                item.set_key('OSM', node['id'])
            item.import_version = node['version']
            item.save()
        api.ChangesetClose()
        return idx+1, None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2012  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin
admin.autodiscover()

from chimere.models import Area
from chimere.feeds import LatestPOIsByCategory, LatestPOIsBySubCategory, \
                          LatestPOIs, LatestPOIsByZone, LatestPOIsByZoneID

def i18n_javascript(request):
    return admin.site.i18n_javascript(request)


urlpatterns = patterns('chimere.views',
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?simple$', 'index', {'simple':True},
        name="simple_index")
)

if settings.CHIMERE_FEEDS:
    urlpatterns += patterns('',
        url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?feeds$', 'chimere.views.rss',
            name='feeds-form'),
        url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?feeds/category/(?P<category_id>\d+)$',
            LatestPOIsByCategory(), name='feeds-cat'),
        url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?feeds/subcategory/(?P<category_id>\d+)$',
            LatestPOIsBySubCategory(), name='feeds-subcat'),
        url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?feeds/global/$', LatestPOIs(),
            name='feeds-global'),
        url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?feeds/area/(?P<area>[0-9-_.]+)$',
            LatestPOIsByZone(), name='feeds-area'),
        url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?feeds/areaid/(?P<area_id>\d+)$',
            LatestPOIsByZoneID(), name='feeds-areaid'),
    )

urlpatterns += patterns('chimere.views',
    url(r'^charte/?$', 'charte', name="charte"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?contact/?$', 'contactus', name="contact"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?edit/$', 'edit',
        name="edit"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?edit/(?P<item_id>\w+)/(?P<submited>\w+)?$',
            'edit', name="edit-item"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?edit-route/$', 'editRoute',
        name="editroute"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?edit-route/(?P<item_id>\w+)/(?P<submited>\w+)?$',
        'editRoute', name="editroute-item"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?getDetail/(?P<marker_id>\d+)/?$', 'getDetail',
        name="get_detail"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?getDescriptionDetail/?(?P<category_id>\d+)/?$',
                        'getDescriptionDetail', name="get_description_detail"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?getGeoObjects/'\
        r'(?P<category_ids>[a-zA-Z0-9_-]+)(/(?P<status>\w+))?$', 'getGeoObjects',
                        name="getgeoobjects"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?getAvailableCategories/$',
                        'get_available_categories', name="get_categories"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]*/)?ty/(?P<tiny_urn>\w+)$',
                        'redirectFromTinyURN', name="tiny"),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?upload_file/((?P<category_id>\w+)/)?$',
                        'uploadFile', name='upload_file'),
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+/)?process_route_file/(?P<file_id>\d+)/$',
                        'processRouteFile', name='process_route_file'),
    # At the end, because it catches large
    url(r'^(?P<area_name>[a-zA-Z0-9_-]+)?', 'index', name="index"),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

VERSION = (2, 0, 1)

def get_version():
    return u'.'.join((unicode(num) for num in VERSION))

__version__ = get_version()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>
#
# RSS : Copyright (C) 2010 Pierre Clarenc <pierre.crc_AT_gmailDOTcom>,
#                          Samuel Renard <renard.samuel_AT_gmailDOTcom>,

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Views of the project
"""

import datetime
from itertools import groupby
import re

from django.conf import settings
from django.core import serializers
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import redirect, render_to_response
from django.template import loader, RequestContext
from django.utils import simplejson
from django.utils.http import urlquote
from django.utils.translation import ugettext as _

from chimere.actions import actions
from chimere.models import Category, SubCategory, PropertyModel, \
     Marker, Route, News, SimpleArea, Area, Color, TinyUrl, RouteFile, \
     get_users_by_area

from chimere.widgets import getMapJS, PointChooserWidget, \
                            RouteChooserWidget, AreaWidget
from chimere.forms import MarkerForm, RouteForm, ContactForm, FileForm, \
     FullFileForm, MultimediaFileFormSet, PictureFileFormSet, notifySubmission,\
     notifyStaff, AreaForm

def get_base_uri(request):
    base_uri = 'http://'
    if 'HTTP_REFERER' in request.META:
        if request.META['HTTP_REFERER'].startswith('https:'):
            base_uri = 'https://'
    if 'SERVER_NAME' in request.META:
        base_uri += request.META['SERVER_NAME']
        if 'SERVER_PORT' in request.META and \
           str(request.META['SERVER_PORT']) != '80':
            base_uri += ":" + str(request.META['SERVER_PORT'])
    return base_uri

#TODO: convert to requestcontext
def get_base_response(area_name=""):
    """
    Get the base url
    """
    base_response_dct = {'media_path':settings.MEDIA_URL,}
    base_url = reverse("chimere:index")
    if not base_url.startswith('/'):
        base_url = '/' + base_url
    if area_name and area_name.endswith('/'):
        area_name = area_name[:-1]
    if area_name:
        base_response_dct['area_name_slash'] = area_name + "/"
        if base_url[-1] != '/':
            base_url += '/'
        base_url += area_name + '/'
    base_response_dct['extra_url'] = base_url
    area = None
    if area_name:
        try:
            area = Area.objects.get(urn=area_name, available=True)
        except ObjectDoesNotExist:
            return None, redirect(reverse('chimere:index'))
    else:
        try:
            area = Area.objects.get(default=True)
            area_name = area.urn
        except ObjectDoesNotExist:
            pass

    base_response_dct['area'] = area
    base_response_dct['area_name'] = area_name
    if area and area.external_css:
        base_response_dct['css_area'] = area.external_css
    base_response_dct['dynamic_categories'] = True \
            if area and area.dynamic_categories else False
    base_response_dct['JQUERY_JS_URLS'] = settings.JQUERY_JS_URLS
    base_response_dct['JQUERY_CSS_URLS'] = settings.JQUERY_CSS_URLS
    return base_response_dct, None

def index(request, area_name=None, default_area=None, simple=False):
    """
    Main page
    """
    # show the news
    # only if user is not came yet today
    today = datetime.date.today().strftime('%y-%m-%d')
    news_visible = False
    if not 'last_visit' in request.session or \
       request.session['last_visit'] != today:
        request.session['last_visit'] = today
        news_visible = True
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    # don't mess with permalink
    zoomout = True
    if request.GET and 'lat' in request.GET \
      and 'lon' in request.GET:
        zoomout = None
    if request.GET and 'current_feature' in request.GET:
        try:
            m = Marker.objects.get(pk=request.GET['current_feature'])
            if m.route:
                response_dct['current_route'] = m.route.pk
        except:
            pass
    response_dct.update({
         'actions':actions, 'action_selected':('view',),
         'error_message':'',
         'news_visible': news_visible,
         'areas_visible': settings.CHIMERE_DISPLAY_AREAS,
         'map_layer':settings.CHIMERE_DEFAULT_MAP_LAYER,
         'dynamic_categories':response_dct['dynamic_categories'],
         'has_default_area':Area.objects.filter(default=True).count(),
         'zoomout':zoomout
        })
    tpl = 'chimere/main_map.html'
    if simple:
        tpl = 'chimere/main_map_simple.html'
    return render_to_response(tpl, response_dct,
                              context_instance=RequestContext(request))

def get_edit_page(redirect_url, item_cls, item_form,
                  multimediafile_formset=MultimediaFileFormSet,
                  picturefile_formset=PictureFileFormSet):
    """
    Edition page
    """
    def func(request, area_name="", item_id=None, cat_type=['M']):
        response_dct, redir = get_base_response(area_name)
        if redir:
            return redir, None, None
        if 'area_name' in response_dct:
            area_name = response_dct['area_name']
        subcategories = SubCategory.getAvailable(cat_type, area_name,
                                                 public=True)
        listed_subcats = []
        if subcategories:
            for cat, subcats in subcategories:
                listed_subcats.append((unicode(cat),
                         [(subcat.pk, subcat.name) for subcat in subcats]))
        # if an item_id is provided: modification
        init_item, ref_item = None, None
        if item_id:
            try:
                init_item = item_cls.objects.get(pk=item_id)
            except:
                return redirect(redirect_url, area_name + '/' if area_name \
                                else ''), None, None
            ref_item = init_item
            modified_item = item_cls.objects.filter(ref_item=init_item,
                               submiter_session_key=request.session.session_key)
            if modified_item.count():
                init_item = modified_item.all()[0]
            response_dct['is_modification'] = True

        init_multi = init_item.get_init_multi() if init_item else None
        init_picture = init_item.get_init_picture() if init_item else None
        if init_item and not request.user.is_superuser and \
                         not init_item.submiter_session_key == \
                                        request.session.session_key:
            # hide personal information
            for k in ('submiter_name', 'submiter_email', 'submiter_comment'):
                setattr(init_item, k, '')
        response_dct['is_superuser'] = request.user.is_superuser
        # If the form has been submited
        if request.method == 'POST':
            inst = None
            # allow to directly modify only if owner or superuser
            if init_item and (request.user.is_superuser or \
                           init_item.submiter_session_key == \
                                        request.session.session_key):
                inst = init_item
            form = item_form(request.POST, request.FILES, instance=inst,
                             subcategories=listed_subcats)
            formset_multi = multimediafile_formset(request.POST, request.FILES,
                                        initial=init_multi, prefix='multimedia')
            formset_picture = picturefile_formset(request.POST, request.FILES,
                                        initial=init_picture, prefix='picture')
            # All validation rules pass
            if form.is_valid() and formset_multi.is_valid() and \
               formset_picture.is_valid():
                item = form.save()
                # set the session key (to permit modifications)
                item.submiter_session_key = request.session.session_key

                # associate to the reference item
                if ref_item:
                    item.ref_item = ref_item
                    if item.pk != ref_item.pk:
                        item.status = 'M'
                        if hasattr(ref_item, 'has_associated_marker'):
                            item.has_associated_marker = \
                                                ref_item.has_associated_marker
                elif not item.ref_item:
                    # initialisation
                    item.ref_item = item

                # just submited
                if not item.status:
                    item.status = 'S'
                item.save()

                marker = item
                if not isinstance(marker, Marker) \
                   and item.associated_marker.count():
                    marker = item.associated_marker.all()[0]
                if marker:
                    # manage multimedia items
                    for f in formset_multi:
                        f.save(marker)

                    for f in formset_picture:
                        f.save(marker)
                base_uri = get_base_uri(request)
                notifySubmission(base_uri, item)
                response_dct = get_base_response(area_name)
                return redirect(redirect_url + '-item',
                            area_name + '/' if area_name else '',
                            item.ref_item.pk, 'submited'), None, subcategories
            else:
                response_dct['error_message'] = _(u"There are missing field(s)"
                                        u" and/or errors in the submited form.")
        else:
            form = item_form(instance=init_item, subcategories=listed_subcats)
            formset_multi = multimediafile_formset(initial=init_multi,
                                                  prefix='multimedia')
            formset_picture = picturefile_formset(initial=init_picture,
                                                 prefix='picture')
        return None, (item_id, init_item, response_dct, form, formset_multi,
                      formset_picture), subcategories
    return func

get_edit_marker = get_edit_page('chimere:edit', Marker, MarkerForm)

def edit(request, area_name="", item_id=None, submited=False):
    """
    Edition page
    """
    response, values, sub_categories = get_edit_marker(request, area_name,
                                                       item_id, ['M', 'B'])
    if response:
        return response
    item_id, init_item, response_dct, form, formset_multi, formset_picture = \
                                                                        values
    # get the "manualy" declared_fields. Ie: properties
    declared_fields = form.declared_fields.keys()
    declared_fields = PropertyModel.objects.filter(available=True).all()
    filtered_properties = PropertyModel.objects.filter(available=True,
                                subcategories__id__isnull=False).all()
    point_value = init_item.point if init_item else None
    if request.POST and request.POST.get('point'):
        point_value = request.POST.get('point')
    response_dct.update({
        'actions':actions,
        'action_selected':('contribute', 'edit'),
        'map_layer':settings.CHIMERE_DEFAULT_MAP_LAYER,
        'form':form,
        'formset_multi':formset_multi,
        'formset_picture':formset_picture,
        'dated':settings.CHIMERE_DAYS_BEFORE_EVENT,
        'extra_head':form.media,
        'marker_id':item_id,
        'sub_categories':sub_categories,
        'point_widget':PointChooserWidget().render('point',
              point_value,
              area_name=response_dct['area_name']),
        'properties':declared_fields,
        'filtered_properties':filtered_properties,
        'submited':submited
    })
    # manualy populate the custom widget
    if 'subcategory' in form.data and form.data['subcategory']:
        response_dct['current_category'] = int(form.data['subcategory'])
    return render_to_response('chimere/edit.html', response_dct,
                              context_instance=RequestContext(request))

def uploadFile(request, category_id='', area_name=''):
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    Form = FileForm if not category_id else FullFileForm
    category = None
    if category_id:
        try:
            category = SubCategory.objects.get(pk=category_id)
            response_dct['category'] = unicode(category)
        except:
            pass
    # If the form has been submited
    if request.method == 'POST':
        form = Form(request.POST, request.FILES)
        # All validation rules pass
        if form.is_valid():
            raw_file = form.cleaned_data['raw_file']
            name = raw_file.name.split('.')[0]
            file_type = raw_file.name.split('.')[-1][0].upper()
            routefile = RouteFile(raw_file=raw_file, name=name,
                                  file_type=file_type)
            routefile.save()
            if not category_id:
                response_dct['gpx_id'] = routefile.pk
                return render_to_response('chimere/upload_file.html',
                         response_dct, context_instance=RequestContext(request))
            routefile.process()
            if not routefile.route:
                response_dct['errors'] = _(u"Bad file. Please check it with an "
                                           u"external software.")
                response_dct.update({'form':form})
                return render_to_response('chimere/upload_file.html',
                         response_dct, context_instance=RequestContext(request))
            route = Route(name=form.cleaned_data['name'], route=routefile.route,
                          associated_file=routefile, status='S')
            route.save()
            route.categories.add(category)
            route.save()
            response_dct['thanks'] = True
            form = Form()
    else:
        # An unbound form
        form = Form()
    response_dct.update({'form':form})
    return render_to_response('chimere/upload_file.html', response_dct,
                              context_instance=RequestContext(request))

def processRouteFile(request, area_name='', file_id=None):
    if file_id:
        try:
            route_file = RouteFile.objects.get(pk=file_id)
            route_file.process()
            route = route_file.route
            if not route:
                return HttpResponse(status=500)
            return HttpResponse('('+simplejson.dumps({'wkt':route,
                                                  'file_id':file_id})+')',
                                'application/javascript', status=200)
        except OSError as e:
            return HttpResponse(e.strerror, status=500)
    else:
        return HttpResponse(status=400)

get_edit_route = get_edit_page('chimere:editroute', Route, RouteForm)

def editRoute(request, area_name="", item_id=None, submited=False):
    """
    Route edition page
    """
    response, values, sub_categories = get_edit_route(request, area_name,
                                                       item_id, ['R', 'B'])
    if response:
        return response
    item_id, init_item, response_dct, form, formset_multi, formset_picture = \
                                                                        values

    # get the "manualy" declared_fields. Ie: properties
    declared_fields = form.declared_fields.keys()
    if 'description' in declared_fields:
        declared_fields.pop(declared_fields.index('description'))
    route_value = init_item.route if init_item else None
    if request.POST and request.POST.get('route'):
        route_value = request.POST.get('route')
    response_dct.update({
        'actions':actions,
        'action_selected':('contribute', 'edit-route'),
        'error_message':'',
        'map_layer':settings.CHIMERE_DEFAULT_MAP_LAYER,
        'form':form,
        'formset_multi':formset_multi,
        'formset_picture':formset_picture,
        'dated':settings.CHIMERE_DAYS_BEFORE_EVENT,
        'extra_head':form.media,
        'sub_categories':sub_categories,
        'route_widget':RouteChooserWidget().render('route', route_value,
                       area_name=response_dct['area_name'], routefile_id='',),
        'properties':declared_fields,
        'submited':submited
    })
    # manualy populate the custom widget
    if 'subcategory' in form.data and form.data['subcategory']:
        response_dct['current_category'] = int(form.data['subcategory'])
    return render_to_response('chimere/edit_route.html', response_dct,
                              context_instance=RequestContext(request))

def submited(request, area_name="", action=""):
    """
    Successful submission page
    """
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    response_dct.update({'actions':actions, 'action_selected':action,})
    return render_to_response('chimere/submited.html', response_dct,
                              context_instance=RequestContext(request))

def charte(request, area_name=""):
    """
    Affichage de la charte
    """
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    response_dct.update({'actions':actions, 'action_selected':('charte',)})
    return render_to_response('chimere/charte.html', response_dct,
                              context_instance=RequestContext(request))

def contactus(request, area_name=""):
    """
    Contact page
    """
    form = None
    msg = ''
    # If the form has been submited
    if request.method == 'POST':
        form = ContactForm(request.POST)
        # All validation rules pass
        if form.is_valid():
            response = notifyStaff(_(u"Comments/request on the map"),
                     form.cleaned_data['content'], form.cleaned_data['email'])
            if response:
                msg = _(u"Thank you for your contribution. It will be taken "\
                        u"into account. If you have left your email you may "\
                        u"be contacted soon for more details.")
            else:
                msg = _(u"Temporary error. Renew your message later.")
    else:
        form = ContactForm()
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    response_dct.update({'actions':actions, 'action_selected':('contact',),
                         'contact_form':form, 'message':msg})
    return render_to_response('chimere/contactus.html', response_dct,
                              context_instance=RequestContext(request))

def getDetail(request, area_name, marker_id):
    '''
    Get the detail for a marker
    '''
    try:
        marker = Marker.objects.filter(id=int(marker_id),
                                       status__in=['A', 'S'])[0]
    except (ValueError, IndexError):
        return HttpResponse('no results')
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    response_dct['marker'] = marker
    if request.method == 'GET':
        if 'simple' in request.GET and request.GET['simple']:
            response_dct['simple'] = True
    parameters = u'current_feature=%s' % marker_id
    parameters += u"&checked_categories=%s" % "_".join([str(m.id) \
                                              for m in marker.categories.all()])
    net_dct = getTinyfiedUrl(request, parameters, response_dct['area_name'])
    share_networks = []
    response_dct['share_url'] = net_dct['url']
    for network in settings.CHIMERE_SHARE_NETWORKS:
        share_networks.append((network[0], network[1] % net_dct, network[2]))
    response_dct['share_networks'] = share_networks
    # to be sure there is unique IDs during a browsing
    response_dct['time_now'] = datetime.datetime.now().strftime('%H%M%S')
    response_dct['dated'] = settings.CHIMERE_DAYS_BEFORE_EVENT \
                            and marker.start_date
    return render_to_response('chimere/detail.html', response_dct,
                              context_instance=RequestContext(request))

def getDescriptionDetail(request, area_name, category_id):
    '''
    Get the description for a category
    '''
    try:
        category = Category.objects.filter(id=int(category_id))[0]
    except (ValueError, IndexError):
        return HttpResponse('no results')
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    response_dct['category'] = category
    return render_to_response('chimere/category_detail.html', response_dct,
                              context_instance=RequestContext(request))

def checkDate(q):
    """
    Filter a queryset to manage dates
    """
    if not settings.CHIMERE_DAYS_BEFORE_EVENT:
        return q
    today = datetime.date.today()
    after = today + datetime.timedelta(settings.CHIMERE_DAYS_BEFORE_EVENT)

    q = q & (  Q(start_date__isnull=True)
             | Q(start_date__gte=today, start_date__lte=after)
             | Q(start_date__lte=today, end_date__gte=today)
            )
    return q

def getGeoObjects(request, area_name, category_ids, status):
    '''
    Get the JSON for markers and routes
    '''
    if not status:
        status = 'A'
    status = status.split('_')
    category_ids = category_ids.split('_')
    try:
        q = checkDate(Q(status__in=status, categories__in=category_ids))
        query = Route.objects.filter(q)
    except:
        return HttpResponse('no results')
    query.order_by('categories')
    routes = list(query)
    jsons = []
    current_cat, colors, idx = None, None, 0
    for route in routes:
        c_cat = route.categories.all()[0]
        if not current_cat or current_cat != c_cat:
            idx = 0
            current_cat = c_cat
            colors = list(Color.objects.filter(color_theme = c_cat.color_theme))
        if colors:
            jsons.append(route.getGeoJSON(color=colors[idx % len(colors)].code))
        else:
            jsons.append(route.getGeoJSON(color='000'))
        idx += 1
    try:
        q = checkDate(Q(status__in=status, categories__in=category_ids))
        query = Marker.objects.filter(q).distinct('pk').order_by('pk')
    except:
        return HttpResponse('no results')
    category_ids = [int(cat_id) for cat_id in category_ids]
    jsons += [geo_object.getGeoJSON(category_ids) for geo_object in list(query)]
    if not jsons:
        return HttpResponse('no results')
    data = '{"type": "FeatureCollection", "features":[%s]}' % ",".join(jsons)
    return HttpResponse(data)

def get_available_categories(request, area_name=None, area=None, status='A',
                             force=None):
    '''
    Get categories for a designed area
    '''
    context_data, redir = get_base_response(area_name)
    area = context_data["area"]
    if redir:
        return redir
    if area and area.dynamic_categories and \
       not "current_extent" in request.GET:
        context_data['sub_categories'] = []
        return render_to_response('chimere/blocks/categories.html', context_data,
                                       context_instance=RequestContext(request))
    if not area or not area.dynamic_categories:
        # Categories are not updated dynamicaly when the user move the map
        # so we return ALL the categories
        subcategories = SubCategory.getAvailable(
                                            area_name=context_data['area_name'])
        context_data['sub_categories'] = subcategories
        return render_to_response('chimere/blocks/categories.html', context_data,
                                       context_instance=RequestContext(request))
    default_message = "<p class='warning'>%s</p>" % \
                                        _("No category available in this area.")
    if not "status" in request.GET: # there must be a status
        status = 'A'
    try:
        status = status.split('_')
        current_extent = request.GET["current_extent"].replace('M', '-')\
                                                      .replace('D', '.')
        area = SimpleArea([float(pt) for pt in current_extent.split('_')])
    except:
        # bad extent format
        return HttpResponse(default_message)
    # if not force and area.isIn(SimpleArea(cookie.AREA):return
    categories = area.getCategories(status, area_name=context_data['area_name'])
    if not categories:
        return HttpResponse(default_message)
    get_cat = lambda subcat: subcat.category
    get_cat_order = lambda subcat: (subcat.category.order, subcat.category,
                                    subcat.order)
    categories = sorted(categories, key=get_cat_order)
    subcategories = [(cat, list(subcats)) \
                               for cat, subcats in groupby(categories, get_cat)]
    context_data['sub_categories'] = subcategories
    return render_to_response('chimere/blocks/categories.html', context_data,
                                       context_instance=RequestContext(request))

def getTinyfiedUrl(request, parameters, area_name=''):
    '''
    Get the tinyfied version of parameters
    '''
    data = {"urn": "", "url":"", "text":""}
    try:
        urn = TinyUrl.getUrnByParameters(parameters)
    except:
        return {}
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    url = reverse('chimere:tiny', args=[(response_dct['area_name'] \
                         if response_dct['area_name'] else '') + '/', urn])
    if not url.startswith('http'):
        url = get_base_uri(request) + url
    url = re.sub("([^:])\/\/", "\g<1>/", url)
    text = settings.PROJECT_NAME
    if 'current_feature' in parameters:
        for item in parameters.split('&'):
            if 'current_feature' in item:
                try:
                    text = unicode(Marker.objects.get(id=item.split('=')[1]))
                except (IndexError, Marker.DoesNotExist):
                    pass
    data["urn"] = urlquote(urn)
    data["url"] = urlquote(url)
    data["text"] = urlquote(text)
    return data

def redirectFromTinyURN(request, area_name='', tiny_urn=''):
    """
    Redirect from a tiny Urn
    """
    parameters = '?' + TinyUrl.getParametersByUrn(tiny_urn)
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    return HttpResponseRedirect(response_dct['extra_url'] + parameters)

def rss(request, area_name=''):
    '''
    Redirect to RSS subscription page
    '''
    response_dct, redir = get_base_response(area_name)
    if redir:
        return redir
    response_dct.update({'actions':actions, 'action_selected':('rss',),
                         'category_rss_feed':'',})
    # If the form has been submited
    if request.method == "POST":
        # User has defined the kind of POI he is interested in : POI in a area
        # (GET method is used for the link with RSS icon in the browser)
        if 'rss_category' in request.POST:
            #User wants to follow all the new POI
            if request.POST['rss_category'] == 'global':
                feeds_link = reverse('chimere:feeds-global')
                return redirect(feeds_link)
            # User wants to follow all the new POI by category or subcategory
            elif request.POST['rss_category'] == 'poi':
                response_dct['category_rss_feed'] = 'category'
                response_dct['sub_categories'] = SubCategory.getAvailable()
                return render_to_response('chimere/feeds/rss.html',
                                  response_dct,
                                  context_instance=RequestContext(request))
            # User wants to follow all the new POI situated in a defined area
            elif request.POST['rss_category'] == 'area':
                # An unbound form
                form = AreaForm()
                area_widget = AreaWidget().render('area', None)
                response_dct.update({
                    'map_layer':settings.CHIMERE_DEFAULT_MAP_LAYER,
                    'extra_head':form.media,
                    'form':form,
                    'category_rss_feed':'area',
                    'area_id':Area.getAvailable(),
                    'area_widget':area_widget
                })
                return render_to_response('chimere/feeds/rss.html',
                                  response_dct,
                                  context_instance=RequestContext(request))
            # Error when submitting the form
            else:
                error = _("Incorrect choice in the list")
                response_dct.update({'error_message':error,
                    'category_rss_feed':'',
                    'sub_categories':SubCategory.getAvailable()})
                return render_to_response('chimere/feeds/rss.html',
                                  response_dct,
                                  context_instance=RequestContext(request))

        # User has specified the category or subcategory he wants to follow =>
        # we redirect him towards the related rss feed
        if 'subcategory' in request.POST and request.POST['subcategory'] != '':
            cat_id = request.POST['subcategory']
            if cat_id.find("cat_") != -1 :
                cat_id = cat_id.split('_')[1]
                feeds_link = reverse('chimere:feeds-cat',
                                     kwargs={'category_id':cat_id})
                return redirect(feeds_link)

            else:
                feeds_link = reverse('chimere:feeds-subcat',
                                     kwargs={'category_id':cat_id})
                return redirect(feeds_link)

        # User has specified the ID of the area he wants to follow
        if 'id_area' in request.POST and request.POST['id_area'] != '':
            feeds_link = reverse('chimere:feeds-areaid',
                                 kwargs={'area_id':request.POST['id_area']})
            return redirect(feeds_link)

        # User has specified the area  he wants to follow => we redirect him
        # towards the related rss feed (using upper left and lower right
        # coordinates)
        elif 'upper_left_lat' in request.POST and \
             request.POST['upper_left_lat'] != '' and \
             'upper_left_lon' in request.POST and \
             request.POST['upper_left_lon'] != '' and \
             'lower_right_lon' in request.POST and \
             request.POST['lower_right_lon'] != '' and \
             'lower_right_lat' in request.POST and \
             request.POST['lower_right_lat'] != '' :
            coords = request.POST['upper_left_lat'] + '_' + \
                     request.POST['upper_left_lon'] + '_' + \
                     request.POST['lower_right_lat'] + '_' + \
                     request.POST['lower_right_lon']
            feeds_link = reverse('chimere:feeds-area',
                                 kwargs={'area':coords})
            return redirect(feeds_link)

    # GET method is used for linking with the RSS icon in the browser when user
    # wants to choose a category to follow
    elif request.method == "GET" and 'rss_category' in request.GET:
        if request.GET['rss_category'] == 'global':
            feeds_link = reverse('chimere:feeds-global')
            return redirect(feeds_link)
        if request.GET['rss_category'] == 'poi':
            response_dct['category_rss_feed'] = 'category'
            response_dct['sub_categories'] = SubCategory.getAvailable(['M','B'])
            return render_to_response('chimere/feeds/rss.html', response_dct,
                                      context_instance=RequestContext(request))
        if request.GET['rss_category'] == 'area':
            # An unbound form
            form = AreaForm()
            response_dct.update({'map_layer':settings.MAP_LAYER,
                            'extra_head':form.media,
                            'form':form,
                            'category_rss_feed':'area',
                            'area_id':Area.getAvailable(),
                            'area_widget':AreaWidget().render('area', None)})
            return render_to_response('chimere/feeds/rss.html', response_dct,
                                      context_instance=RequestContext(request))

    # User access to the RSS tab
    else:
        return render_to_response('chimere/feeds/rss.html', response_dct,
                                  context_instance=RequestContext(request))



#-*- coding: utf-8 -*-

###########################################################################
##                                                                       ##
## Copyrights Etienne Chové <chove@crans.org> 2009-2010                  ##
##                                                                       ##
## This program is free software: you can redistribute it and/or modify  ##
## it under the terms of the GNU General Public License as published by  ##
## the Free Software Foundation, either version 3 of the License, or     ##
## (at your option) any later version.                                   ##
##                                                                       ##
## This program is distributed in the hope that it will be useful,       ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of        ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         ##
## GNU General Public License for more details.                          ##
##                                                                       ##
## You should have received a copy of the GNU General Public License     ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>. ##
##                                                                       ##
###########################################################################

## HomePage : http://wiki.openstreetmap.org/wiki/PythonOsmApi

###########################################################################
## History                                                               ##
###########################################################################
## 0.2.19  2010-05-24 Add debug message on ApiError                      ##
## 0.2.18  2010-04-20 Fix ChangesetClose and _http_request               ##
## 0.2.17  2010-01-02 Capabilities implementation                        ##
## 0.2.16  2010-01-02 ChangesetsGet by Alexander Rampp                   ##
## 0.2.15  2009-12-16 xml encoding error for < and >                     ##
## 0.2.14  2009-11-20 changesetautomulti parameter                       ##
## 0.2.13  2009-11-16 modify instead update for osc                      ##
## 0.2.12  2009-11-14 raise ApiError on 4xx errors -- Xoff               ##
## 0.2.11  2009-10-14 unicode error on ChangesetUpload                   ##
## 0.2.10  2009-10-14 RelationFullRecur definition                       ##
## 0.2.9   2009-10-13 automatic changeset management                     ##
##                    ChangesetUpload implementation                     ##
## 0.2.8   2009-10-13 *(Create|Update|Delete) use not unique _do method  ##
## 0.2.7   2009-10-09 implement all missing fonctions except             ##
##                    ChangesetsGet and GetCapabilities                  ##
## 0.2.6   2009-10-09 encoding clean-up                                  ##
## 0.2.5   2009-10-09 implements NodesGet, WaysGet, RelationsGet         ##
##                               ParseOsm, ParseOsc                      ##
## 0.2.4   2009-10-06 clean-up                                           ##
## 0.2.3   2009-09-09 keep http connection alive for multiple request    ##
##                    (Node|Way|Relation)Get return None when object     ##
##                    have been deleted (raising error before)           ##
## 0.2.2   2009-07-13 can identify applications built on top of the lib  ##
## 0.2.1   2009-05-05 some changes in constructor -- chove@crans.org     ##
## 0.2     2009-05-01 initial import                                     ##
###########################################################################

__version__ = '0.2.19'

import httplib, base64, xml.dom.minidom, time, sys, urllib

class ApiError(Exception):
    	
    def __init__(self, status, reason, payload):
        self.status  = status
        self.reason  = reason
        self.payload = payload
    
    def __str__(self):
        return "Request failed: " + str(self.status) + " - " + self.reason + " - " + self.payload

###########################################################################
## Main class                                                            ##

class OsmApi:
        
    def __init__(self,
        username = None,
        password = None,
        passwordfile = None,
        appid = "",
        created_by = "PythonOsmApi/"+__version__,
        api = "www.openstreetmap.org",
        changesetauto = False,
        changesetautotags = {},
        changesetautosize = 500,
        changesetautomulti = 1,
        debug = False
        ):
    
        # debug
        self._debug = debug
        
        # Get username
        if username:
            self._username = username
        elif passwordfile:
            self._username =  open(passwordfile).readline().split(":")[0].strip()            
    
        # Get password
        if password:
            self._password   = password
        elif passwordfile:
            for l in open(passwordfile).readlines():
                l = l.strip().split(":")
                if l[0] == self._username:
                    self._password = l[1]

        # Changest informations
        self._changesetauto      = changesetauto      # auto create and close changesets
        self._changesetautotags  = changesetautotags  # tags for automatic created changesets
        self._changesetautosize  = changesetautosize  # change count for auto changeset
        self._changesetautosize  = changesetautosize  # change count for auto changeset
        self._changesetautomulti = changesetautomulti # close a changeset every # upload
        self._changesetautocpt   = 0
        self._changesetautodata  = []                 # data to upload for auto group
        
        # Get API
        self._api = api

        # Get created_by
        if not appid:
            self._created_by = created_by
        else:
            self._created_by = appid + " (" + created_by + ")"

        # Initialisation     
        self._CurrentChangesetId = 0
        
        # Http connection
        self._conn = httplib.HTTPConnection(self._api, 80)

    def __del__(self):
        if self._changesetauto:
            self._changesetautoflush(True)
        return None

    #######################################################################
    # Capabilities                                                        #
    #######################################################################
    
    def Capabilities(self):
        """ Returns ApiCapabilities. """
        uri = "/api/capabilities"
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        print data.getElementsByTagName("osm")
        data = data.getElementsByTagName("osm")[0].getElementsByTagName("api")[0]
        result = {}
        for elem in data.childNodes:
            if elem.nodeType <> elem.ELEMENT_NODE:
                continue
            result[elem.nodeName] = {}
            print elem.nodeName
            for k, v in elem.attributes.items():
                try:
                    result[elem.nodeName][k] = float(v)
                except:
                    result[elem.nodeName][k] = v
        return result

    #######################################################################
    # Node                                                                #
    #######################################################################

    def NodeGet(self, NodeId, NodeVersion = -1):
        """ Returns NodeData for node #NodeId. """
        uri = "/api/0.6/node/"+str(NodeId)
        if NodeVersion <> -1: uri += "/"+str(NodeVersion)
        data = self._get(uri)
        if not data: return data
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osm")[0].getElementsByTagName("node")[0]
        return self._DomParseNode(data)

    def NodeCreate(self, NodeData):
        """ Creates a node. Returns updated NodeData (without timestamp). """
        return self._do("create", "node", NodeData)
            
    def NodeUpdate(self, NodeData):
        """ Updates node with NodeData. Returns updated NodeData (without timestamp). """
        return self._do("modify", "node", NodeData)

    def NodeDelete(self, NodeData):
        """ Delete node with NodeData. Returns updated NodeData (without timestamp). """
        return self._do("delete", "node", NodeData)

    def NodeHistory(self, NodeId):
        """ Returns dict(NodeVerrsion: NodeData). """
        uri = "/api/0.6/node/"+str(NodeId)+"/history"
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = {}
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("node"):
            data = self._DomParseNode(data)
            result[data[u"version"]] = data
        return result

    def NodeWays(self, NodeId):
        """ Returns [WayData, ... ] containing node #NodeId. """
        uri = "/api/0.6/node/%d/ways"%NodeId
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = []
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("way"):
            data = self._DomParseRelation(data)
            result.append(data)
        return result
    
    def NodeRelations(self, NodeId):
        """ Returns [RelationData, ... ] containing node #NodeId. """
        uri = "/api/0.6/node/%d/relations"%NodeId
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = []
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("relation"):
            data = self._DomParseRelation(data)
            result.append(data)
        return result

    def NodesGet(self, NodeIdList):
        """ Returns dict(NodeId: NodeData) for each node in NodeIdList """
        uri  = "/api/0.6/nodes?nodes=" + ",".join([str(x) for x in NodeIdList])
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = {}
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("node"):
            data = self._DomParseNode(data)
            result[data[u"id"]] = data
        return result

    #######################################################################
    # Way                                                                 #
    #######################################################################

    def WayGet(self, WayId, WayVersion = -1):
        """ Returns WayData for way #WayId. """
        uri = "/api/0.6/way/"+str(WayId)
        if WayVersion <> -1: uri += "/"+str(WayVersion)
        data = self._get(uri)
        if not data: return data
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osm")[0].getElementsByTagName("way")[0]
        return self._DomParseWay(data)
    
    def WayCreate(self, WayData):
        """ Creates a way. Returns updated WayData (without timestamp). """
        return self._do("create", "way", WayData)

    def WayUpdate(self, WayData):
        """ Updates way with WayData. Returns updated WayData (without timestamp). """
        return self._do("modify", "way", WayData)

    def WayDelete(self, WayData):
        """ Delete way with WayData. Returns updated WayData (without timestamp). """
        return self._do("delete", "way", WayData)

    def WayHistory(self, WayId):
        """ Returns dict(WayVerrsion: WayData). """
        uri = "/api/0.6/way/"+str(WayId)+"/history"
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = {}
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("way"):
            data = self._DomParseWay(data)
            result[data[u"version"]] = data
        return result
    
    def WayRelations(self, WayId):
        """ Returns [RelationData, ...] containing way #WayId. """
        uri = "/api/0.6/way/%d/relations"%WayId
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = []
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("relation"):
            data = self._DomParseRelation(data)
            result.append(data)
        return result

    def WayFull(self, WayId):
        """ Return full data for way WayId as list of {type: node|way|relation, data: {}}. """
        uri = "/api/0.6/way/"+str(WayId)+"/full"
        data = self._get(uri)
        return self.ParseOsm(data)

    def WaysGet(self, WayIdList):
        """ Returns dict(WayId: WayData) for each way in WayIdList """
        uri = "/api/0.6/ways?ways=" + ",".join([str(x) for x in WayIdList])
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = {}
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("way"):
            data = self._DomParseWay(data)
            result[data[u"id"]] = data
        return result

    #######################################################################
    # Relation                                                            #
    #######################################################################

    def RelationGet(self, RelationId, RelationVersion = -1):
        """ Returns RelationData for relation #RelationId. """
        uri = "/api/0.6/relation/"+str(RelationId)
        if RelationVersion <> -1: uri += "/"+str(RelationVersion)
        data = self._get(uri)
        if not data: return data
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osm")[0].getElementsByTagName("relation")[0]
        return self._DomParseRelation(data)

    def RelationCreate(self, RelationData):
        """ Creates a relation. Returns updated RelationData (without timestamp). """
        return self._do("create", "relation", RelationData)
    
    def RelationUpdate(self, RelationData):
        """ Updates relation with RelationData. Returns updated RelationData (without timestamp). """
        return self._do("modify", "relation", RelationData)

    def RelationDelete(self, RelationData):
        """ Delete relation with RelationData. Returns updated RelationData (without timestamp). """
        return self._do("delete", "relation", RelationData)

    def RelationHistory(self, RelationId):
        """ Returns dict(RelationVerrsion: RelationData). """
        uri = "/api/0.6/relation/"+str(RelationId)+"/history"
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = {}
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("relation"):
            data = self._DomParseRelation(data)
            result[data[u"version"]] = data
        return result
    
    def RelationRelations(self, RelationId):
        """ Returns list of RelationData containing relation #RelationId. """
        uri = "/api/0.6/relation/%d/relations"%RelationId
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = []
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("relation"):
            data = self._DomParseRelation(data)
            result.append(data)
        return result

    def RelationFullRecur(self, RelationId):
        """ Return full data for relation RelationId. Recurisve version relation of relations. """
        data = []
        todo = [RelationId]
        done = []
        while todo:
            rid = todo.pop(0)
            done.append(rid)
            temp = self.RelationFull(rid)
            for item in temp:
                if item["type"] <> "relation":
                    continue
                if item["data"]["id"] in done:
                    continue
                todo.append(item["data"]["id"])            
            data += temp
        return data
    
    def RelationFull(self, RelationId):
        """ Return full data for relation RelationId as list of {type: node|way|relation, data: {}}. """
        uri = "/api/0.6/relation/"+str(RelationId)+"/full"
        data = self._get(uri)
        return self.ParseOsm(data)

    def RelationsGet(self, RelationIdList):
        """ Returns dict(RelationId: RelationData) for each relation in RelationIdList """
        uri = "/api/0.6/relations?relations=" + ",".join([str(x) for x in RelationIdList])
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        result = {}
        for data in data.getElementsByTagName("osm")[0].getElementsByTagName("relation"):
            data = self._DomParseRelation(data)
            result[data[u"id"]] = data            
        return result

    #######################################################################
    # Changeset                                                           #
    #######################################################################

    def ChangesetGet(self, ChangesetId):
        """ Returns ChangesetData for changeset #ChangesetId. """
        data = self._get("/api/0.6/changeset/"+str(ChangesetId))
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osm")[0].getElementsByTagName("changeset")[0]
        return self._DomParseChangeset(data)
    
    def ChangesetUpdate(self, ChangesetTags = {}):
        """ Updates current changeset with ChangesetTags. """
        if self._CurrentChangesetId == -1:
            raise Exception, "No changeset currently opened"
        if u"created_by" not in ChangesetTags:
            ChangesetTags[u"created_by"] = self._created_by
        result = self._put("/api/0.6/changeset/"+str(self._CurrentChangesetId), self._XmlBuild("changeset", {u"tag": ChangesetTags}))
        return self._CurrentChangesetId

    def ChangesetCreate(self, ChangesetTags = {}):
        """ Opens a changeset. Returns #ChangesetId. """
        if self._CurrentChangesetId:
            raise Exception, "Changeset alreadey opened"
        if u"created_by" not in ChangesetTags:
            ChangesetTags[u"created_by"] = self._created_by
        result = self._put("/api/0.6/changeset/create", self._XmlBuild("changeset", {u"tag": ChangesetTags}))
        self._CurrentChangesetId = int(result)
        return self._CurrentChangesetId
    
    def ChangesetClose(self):
        """ Closes current changeset. Returns #ChangesetId. """
        if not self._CurrentChangesetId:
            raise Exception, "No changeset currently opened"
        result = self._put("/api/0.6/changeset/"+str(self._CurrentChangesetId)+"/close", u"")
        CurrentChangesetId = self._CurrentChangesetId
        self._CurrentChangesetId = 0
        return CurrentChangesetId

    def ChangesetUpload(self, ChangesData):
        """ Upload data. ChangesData is a list of dict {type: node|way|relation, action: create|delete|modify, data: {}}. Returns list with updated ids. """
        data = ""
        data += u"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
        data += u"<osmChange version=\"0.6\" generator=\"" + self._created_by + "\">\n"
        for change in ChangesData:
            data += u"<"+change["action"]+">\n"
            change["data"]["changeset"] = self._CurrentChangesetId
            data += self._XmlBuild(change["type"], change["data"], False).decode("utf-8")
            data += u"</"+change["action"]+">\n"
        data += u"</osmChange>"
        data = self._http("POST", "/api/0.6/changeset/"+str(self._CurrentChangesetId)+"/upload", True, data.encode("utf-8"))
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("diffResult")[0]
        data = [x for x in data.childNodes if x.nodeType == x.ELEMENT_NODE]
        for i in range(len(ChangesData)):
            if ChangesData[i]["action"] == "delete":
                ChangesData[i]["data"].pop("version")
            else:
                ChangesData[i]["data"]["version"] = int(data[i].getAttribute("new_id"))
        return ChangesData
        
    def ChangesetDownload(self, ChangesetId):
        """ Download data from a changeset. Returns list of dict {type: node|way|relation, action: create|delete|modify, data: {}}. """
        uri = "/api/0.6/changeset/"+str(ChangesetId)+"/download"
        data = self._get(uri)
        return self.ParseOsc(data)
    
    def ChangesetsGet(self, min_lon=None, min_lat=None, max_lon=None, max_lat=None,
                      userid=None, username=None,
                      closed_after=None, created_before=None,
                      only_open=False, only_closed=False):
        """ Returns dict(ChangsetId: ChangesetData) matching all criteria. """
        
        uri = "/api/0.6/changesets"
        params = {}
        if min_lon or min_lat or max_lon or max_lat:
            params["bbox"] = ",".join([str(min_lon),str(min_lat),str(max_lon),str(max_lat)])
        if userid:
            params["user"] = userid
        if username:
            params["display_name"] = username
        if closed_after and not created_before:
            params["time"] = closed_after
        if created_before:
            if not closed_after:
                closed_after = "1970-01-01T00:00:00Z"
            params["time"] = closed_after + "," + created_before
        if only_open:
            params["open"] = 1
        if only_closed:
            params["closed"] = 1
            
        if params:
            uri += "?" + urllib.urlencode(params)
                
        data = self._get(uri)
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osm")[0].getElementsByTagName("changeset")
        result = {}
        for curChangeset in data:
            tmpCS = self._DomParseChangeset(curChangeset)
            result[tmpCS["id"]] = tmpCS
        return result
    
    #######################################################################
    # Other                                                               #
    #######################################################################

    def Map(self, min_lon, min_lat, max_lon, max_lat):
        """ Download data in bounding box. Returns list of dict {type: node|way|relation, data: {}}. """
        uri = "/api/0.6/map?bbox=%f,%f,%f,%f"%(min_lon, min_lat, max_lon, max_lat)
        data = self._get(uri)
        return self.ParseOsm(data)

    #######################################################################
    # Data parser                                                         #
    #######################################################################
    
    def ParseOsm(self, data):
        """ Parse osm data. Returns list of dict {type: node|way|relation, data: {}}. """
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osm")[0]
        result = []
        for elem in data.childNodes:
            if elem.nodeName == u"node":
                result.append({u"type": elem.nodeName, u"data": self._DomParseNode(elem)})
            elif elem.nodeName == u"way":
                result.append({u"type": elem.nodeName, u"data": self._DomParseWay(elem)})                        
            elif elem.nodeName == u"relation":
                result.append({u"type": elem.nodeName, u"data": self._DomParseRelation(elem)})
        return result    

    def ParseOsc(self, data):
        """ Parse osc data. Returns list of dict {type: node|way|relation, action: create|delete|modify, data: {}}. """
        data = xml.dom.minidom.parseString(data)
        data = data.getElementsByTagName("osmChange")[0]
        result = []
        for action in data.childNodes:
            if action.nodeName == u"#text": continue
            for elem in action.childNodes:
                if elem.nodeName == u"node":
                    result.append({u"action":action.nodeName, u"type": elem.nodeName, u"data": self._DomParseNode(elem)})
                elif elem.nodeName == u"way":
                    result.append({u"action":action.nodeName, u"type": elem.nodeName, u"data": self._DomParseWay(elem)})                        
                elif elem.nodeName == u"relation":
                    result.append({u"action":action.nodeName, u"type": elem.nodeName, u"data": self._DomParseRelation(elem)})
        return result

    #######################################################################
    # Internal http function                                              #
    #######################################################################

    def _do(self, action, OsmType, OsmData):
        if self._changesetauto:
            self._changesetautodata.append({"action":action, "type":OsmType, "data":OsmData})
            self._changesetautoflush()
            return None
        else:
            return self._do_manu(action, OsmType, OsmData)
            
    def _do_manu(self, action, OsmType, OsmData):        
        if not self._CurrentChangesetId:
            raise Exception, "You need to open a changeset before uploading data"
        if u"timestamp" in OsmData:
            OsmData.pop(u"timestamp")
        OsmData[u"changeset"] = self._CurrentChangesetId
        if action == "create":
            if OsmData.get(u"id", -1) > 0:
                raise Exception, "This "+OsmType+" already exists"
            result = self._put("/api/0.6/"+OsmType+"/create", self._XmlBuild(OsmType, OsmData))
            OsmData[u"id"] = int(result.strip())
            OsmData[u"version"] = 1
            return OsmData
        elif action == "modify":
            result = self._put("/api/0.6/"+OsmType+"/"+str(OsmData[u"id"]), self._XmlBuild(OsmType, OsmData))
            OsmData[u"version"] = int(result.strip())
            return OsmData
        elif action =="delete":
            result = self._delete("/api/0.6/"+OsmType+"/"+str(OsmData[u"id"]), self._XmlBuild(OsmType, OsmData))
            OsmData[u"version"] = int(result.strip())
            OsmData[u"visible"] = False
            return OsmData
    
    def flush(self):
        return self._changesetautoflush(True)
        
    def _changesetautoflush(self, force = False):
        while (len(self._changesetautodata) >= self._changesetautosize) or (force and self._changesetautodata):
            if self._changesetautocpt == 0:
                self.ChangesetCreate(self._changesetautotags)
            self.ChangesetUpload(self._changesetautodata[:self._changesetautosize])
            self._changesetautodata = self._changesetautodata[self._changesetautosize:]
            self._changesetautocpt += 1
            if self._changesetautocpt == self._changesetautomulti:
                self.ChangesetClose()
                self._changesetautocpt = 0
        if self._changesetautocpt and force:
            self.ChangesetClose()
            self._changesetautocpt = 0
        return None
        
    def _http_request(self, cmd, path, auth, send):
        if self._debug:
            path2 = path
            if len(path2) > 50:
                path2 = path2[:50]+"[...]"
            print >>sys.stderr, "%s %s %s"%(time.strftime("%Y-%m-%d %H:%M:%S"),cmd,path2)
        self._conn.putrequest(cmd, path)
        self._conn.putheader('User-Agent', self._created_by)
        if auth:
            self._conn.putheader('Authorization', 'Basic ' + base64.encodestring(self._username + ':' + self._password).strip())
        if send <> None:
            self._conn.putheader('Content-Length', len(send))
        self._conn.endheaders()
        if send:
            self._conn.send(send)
        response = self._conn.getresponse()
        if response.status <> 200:
            payload = response.read().strip()
            if response.status == 410:
                return None
            raise ApiError(response.status, response.reason, payload)
        if self._debug:
            print >>sys.stderr, "%s %s %s done"%(time.strftime("%Y-%m-%d %H:%M:%S"),cmd,path2)
        return response.read()
    
    def _http(self, cmd, path, auth, send):
        i = 0
        while True:
            i += 1
            try:
                return self._http_request(cmd, path, auth, send)
            except ApiError, e:
                if e.status >= 500:
                    if i == 5: raise
                    if i <> 1: time.sleep(5)
                    self._conn = httplib.HTTPConnection(self._api, 80)
                else: raise
            except Exception:
                if i == 5: raise
                if i <> 1: time.sleep(5)
                self._conn = httplib.HTTPConnection(self._api, 80)
    
    def _get(self, path):
        return self._http('GET', path, False, None)

    def _put(self, path, data):
        return self._http('PUT', path, True, data)
    
    def _delete(self, path, data):
        return self._http('DELETE', path, True, data)
    
    #######################################################################
    # Internal dom function                                               #
    #######################################################################
    
    def _DomGetAttributes(self, DomElement):
        """ Returns a formated dictionnary of attributes of a DomElement. """
        result = {}
        for k, v in DomElement.attributes.items():
            if k == u"uid"         : v = int(v)
            elif k == u"changeset" : v = int(v)
            elif k == u"version"   : v = int(v)
            elif k == u"id"        : v = int(v)
            elif k == u"lat"       : v = float(v)
            elif k == u"lon"       : v = float(v)
            elif k == u"open"      : v = v=="true"
            elif k == u"visible"   : v = v=="true"
            elif k == u"ref"       : v = int(v)
            result[k] = v
        return result            
        
    def _DomGetTag(self, DomElement):
        """ Returns the dictionnary of tags of a DomElement. """
        result = {}
        for t in DomElement.getElementsByTagName("tag"):
            k = t.attributes["k"].value
            v = t.attributes["v"].value
            result[k] = v
        return result

    def _DomGetNd(self, DomElement):
        """ Returns the list of nodes of a DomElement. """
        result = []
        for t in DomElement.getElementsByTagName("nd"):
            result.append(int(int(t.attributes["ref"].value)))
        return result            

    def _DomGetMember(self, DomElement):
        """ Returns a list of relation members. """
        result = []
        for m in DomElement.getElementsByTagName("member"):
            result.append(self._DomGetAttributes(m))
        return result

    def _DomParseNode(self, DomElement):
        """ Returns NodeData for the node. """
        result = self._DomGetAttributes(DomElement)
        result[u"tag"] = self._DomGetTag(DomElement)
        return result

    def _DomParseWay(self, DomElement):
        """ Returns WayData for the way. """
        result = self._DomGetAttributes(DomElement)
        result[u"tag"] = self._DomGetTag(DomElement)
        result[u"nd"]  = self._DomGetNd(DomElement)        
        return result
    
    def _DomParseRelation(self, DomElement):
        """ Returns RelationData for the relation. """
        result = self._DomGetAttributes(DomElement)
        result[u"tag"]    = self._DomGetTag(DomElement)
        result[u"member"] = self._DomGetMember(DomElement)
        return result

    def _DomParseChangeset(self, DomElement):
        """ Returns ChangesetData for the changeset. """
        result = self._DomGetAttributes(DomElement)
        result[u"tag"] = self._DomGetTag(DomElement)
        return result

    #######################################################################
    # Internal xml builder                                                #
    #######################################################################

    def _XmlBuild(self, ElementType, ElementData, WithHeaders = True):

        xml  = u""
        if WithHeaders:
            xml += u"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            xml += u"<osm version=\"0.6\" generator=\"" + self._created_by + "\">\n"

        # <element attr="val">
        xml += u"  <" + ElementType
        if u"id" in ElementData:
            xml += u" id=\"" + str(ElementData[u"id"]) + u"\""        
        if u"lat" in ElementData:
            xml += u" lat=\"" + str(ElementData[u"lat"]) + u"\""        
        if u"lon" in ElementData:
            xml += u" lon=\"" + str(ElementData[u"lon"]) + u"\""
        if u"version" in ElementData:
            xml += u" version=\"" + str(ElementData[u"version"]) + u"\""
        xml += u" visible=\"" + str(ElementData.get(u"visible", True)).lower() + u"\""
        if ElementType in [u"node", u"way", u"relation"]:
            xml += u" changeset=\"" + str(self._CurrentChangesetId) + u"\""
        xml += u">\n"

        # <tag... />
        for k, v in ElementData.get(u"tag", {}).items():
            xml += u"    <tag k=\""+self._XmlEncode(k)+u"\" v=\""+self._XmlEncode(v)+u"\"/>\n"

        # <member... />
        for member in ElementData.get(u"member", []):
            xml += u"    <member type=\""+member[u"type"]+"\" ref=\""+str(member[u"ref"])+u"\" role=\""+self._XmlEncode(member[u"role"])+"\"/>\n"

        # <nd... />
        for ref in ElementData.get(u"nd", []):
            xml += u"    <nd ref=\""+str(ref)+u"\"/>\n"

        # </element>
        xml += u"  </" + ElementType + u">\n"
        
        if WithHeaders:
            xml += u"</osm>\n"

        return xml.encode("utf8")

    def _XmlEncode(self, text):
        return text.replace("&", "&amp;").replace("\"", "&quot;").replace("<","&lt;").replace(">","&gt;")

## End of main class                                                     ##
###########################################################################



# -*- coding: utf-8 -*-
import sys

from django.contrib.gis.db import models

from logging import getLogger
from datetime import datetime, date, timedelta

from django.conf import settings

# BASE
class BaseGeoQuerySet(models.query.QuerySet):
    """
    Custom queryset for Geo objects.
    """
    def available(self):
        """ Chainable filter to retrieve only available elements. """
        return self.filter(status="A")
    
    def submited(self):
        """ Chainable filter to retrieve only submited elements. """
        return self.filter(status="S")
    
    def active(self):
        """
        Returns objects where within the active date period of events.
        """
        # Fields are not set in the model in this case
        if not settings.CHIMERE_DAYS_BEFORE_EVENT:
            return self
        today = date.today()
        after = today + timedelta(settings.CHIMERE_DAYS_BEFORE_EVENT)
        return self.filter(end_date__gte=today, start_date__lte=after)
    
class BaseGeoManager(models.GeoManager):
    """
    Custom base manager for Geo objects.
    """
    def get_query_set(self):
        """ Use our custom QuerySet. """
        return BaseGeoQuerySet(self.model)
    
    # Methods defined in the queryset that we also want exposed in the
    # It would be nice to implement it with a magic method instead, later :)
    def available(self):
        return self.get_query_set().available()
    
    def submited(self):
        return self.get_query_set().submited()
    
    def active(self):
        return self.get_query_set().active()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import csv
import datetime, time

from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ObjectDoesNotExist

from chimere import tasks
from chimere.admin import export_to_csv, export_to_kml, export_to_shapefile
from chimere.models import Category, SubCategory, Marker, Route

EXPORTER = {'CSV':export_to_csv,
            'KML':export_to_kml,
            'SHP':export_to_shapefile}

class Command(BaseCommand):
    args = '<subcategory_id> <CSV|KML|SHP> <marker|route> <filename>'
    help = "Export items from a subcategory in CSV, KML or ShapeFile"

    def handle(self, *args, **options):
        subcat = None
        if args and args[0]:
            try:
                subcat = SubCategory.objects.get(pk=int(args[0]))
            except (ValueError, ObjectDoesNotExist) as e:
                raise CommandError("Sub-category with ID '%s' doesn't exist." %\
                                                                        args[0])
        while not subcat:
            self.stdout.write('Choose a sub-category:\n')
            for cat in Category.objects.order_by('order'):
                self.stdout.write('* %s\n' % cat.name)
                for subcat in cat.subcategories.order_by('order').all():
                    self.stdout.write(' %d - %s\n' % (subcat.pk, subcat.name))
            self.stdout.write('\nSub-category ID: ')
            self.stdout.flush()
            v = raw_input()
            try:
                subcat = SubCategory.objects.get(pk=v)
            except (ValueError, ObjectDoesNotExist) as e:
                subcat = None
                self.stdout.write("Sub-category with ID '%s' doesn't exist.\n\n"
                                  % v)
        frmat = None
        if args and args[1]:
            try:
                assert(args[1] in ('CSV', 'KML', 'SHP'))
            except AssertionError:
                raise CommandError("'%s' format is not managed." % args[1])
            frmat = args[1]
        while frmat not in ('CSV', 'KML', 'SHP'):
            self.stdout.write('Choose a format (CSV, KML or SHP): ')
            frmat = raw_input().replace('\n', '')
        exporter = EXPORTER[frmat]
        cls = None
        if args and args[2]:
            try:
                assert(args[2] in ('marker', 'route'))
            except AssertionError:
                raise CommandError("Exported item must be 'marker' or 'route'."
                                   % args[1])
            if args[2] == 'marker':
                cls = Marker
            elif args[2] == 'route':
                cls = Route
        while not cls:
            self.stdout.write('Choose an item type:\n 1 - marker\n 2 - route\n')
            self.stdout.write('Number: ')
            v = raw_input()
            if v == '1':
                cls = Marker
            elif v == '2':
                cls = Route
        filename = ''
        if args and args[3]:
            filename = args[3]
        else:
            self.stdout.write('Filename: ')
            filename = raw_input()
        response = exporter(None, None, cls.objects.filter(categories=subcat))
        try:
            with open(filename, 'w+') as fl:
                fl.write(response.content)
        except IOError as e:
            raise CommandError(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2012  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Forms
"""
from django import forms
from django.conf import settings
from django.contrib.gis.db import models
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.forms.formsets import formset_factory
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User, Permission, ContentType
from django.contrib.admin.widgets import AdminDateWidget
from django.core.mail import EmailMessage, BadHeaderError

from chimere.models import Marker, Route, PropertyModel, Property, Area,\
   News, Category, SubCategory, RouteFile, MultimediaFile, MultimediaType, \
   PictureFile, Importer, IMPORTER_CHOICES, IFRAME_LINKS, MultimediaExtension
from chimere.widgets import AreaField, PointField, TextareaWidget, \
    ImportFiltrWidget, TextareaAdminWidget, DatePickerWidget, \
    ImporterChoicesWidget, RE_XAPI

from datetime import timedelta, datetime, tzinfo

ZERO = timedelta(0)

class UTC(tzinfo):
    """UTC time zone"""

    def utcoffset(self, dt):
        return ZERO

    def tzname(self, dt):
        return settings.TIME_ZONE

    def dst(self, dt):
        return ZERO

def notifyStaff(subject, body, sender=None):
    if not settings.EMAIL_HOST:
        return
    if settings.PROJECT_NAME:
        subject = u'[%s] %s' % (settings.PROJECT_NAME, subject)
    user_list = [u.email for u in
          User.objects.filter(is_staff=True).exclude(email="").order_by('id')]
    headers = {}
    if sender:
        headers['Reply-To'] = sender
    email = EmailMessage(subject, body, user_list[0], user_list,
                         headers=headers)
    try:
        email.send()
    except BadHeaderError:
        return False
    return True

def notifySubmission(absolute_uri, geo_object):
    category = u" - ".join([unicode(cat) for cat in geo_object.categories.all()])
    subject = u'%s %s' % (_(u"New submission for"), category)
    message = _(u'The new item "%s" has been submited in the category: ') % \
              geo_object.name + category
    message += "\n\n" + _(u"To valid, precise or unvalid this item: ")
    named_url = 'admin:chimere_%s_change'% geo_object.__class__.__name__.lower()
    message += absolute_uri + reverse(named_url, args=(geo_object.pk,))
    message += u"\n\n--\nChimère"
    return notifyStaff(subject, message)

class ContactForm(forms.Form):
    """
    Main form for categories
    """
    email = forms.EmailField(label=_("Email (optional)"), required=False)
    content = forms.CharField(label=_("Object"), widget=forms.Textarea)

class OSMForm(forms.Form):
    """
    OSM export form
    """
    username = forms.CharField(label=_("OSM user"))
    password = forms.CharField(label=_(u"Password"),
                               widget=forms.PasswordInput(render_value=False))
    # API URL are hardcoded: the day the API change Chimère will need
    # adaptations not only on this portion...
    api = forms.ChoiceField(label=_(u"API"),
            choices=(('', '--'),
                     ('api06.dev.openstreetmap.org',
                      _(u"Test API - %s") % 'api06.dev.openstreetmap.org'),
                     ('api.openstreetmap.org/api',
                      _(u"Main API - %s") % 'api.openstreetmap.org/api'),
                     ))

class NewsAdminForm(forms.ModelForm):
    """
    Main form for news
    """
    content = forms.CharField(widget=TextareaAdminWidget)
    class Meta:
        model = News

class ImporterAdminForm(forms.ModelForm):
    filtr = forms.CharField(widget=ImportFiltrWidget, required=False)
    importer_type = forms.ChoiceField(widget=ImporterChoicesWidget,
                                    choices=[('', '--')]+list(IMPORTER_CHOICES))
    default_description = forms.CharField(widget=TextareaAdminWidget,
                                          required=False)
    class Meta:
        model = Importer
        widgets = {
            'source': forms.TextInput(attrs={'size': 80}),
            'filtr': forms.TextInput(attrs={'size': 80}),
        }

    def clean(self):
        '''
        Verify that only one type of source is provided
        Verify that shapefiles are zipped
        '''
        if self.cleaned_data.get('importer_type') == 'OSM' and \
           not self.cleaned_data.get('filtr'):
            raise forms.ValidationError(_(u"For OSM import you must be "\
                      u"provide a filter. Select an area and node/way filter."))
        if self.cleaned_data.get('importer_type') == 'OSM' and \
           not RE_XAPI.match(self.cleaned_data.get('filtr')):
            raise forms.ValidationError(_(u"For OSM import you must be "\
                      u"provide a filter. Select an area and node/way filter."))
        if self.cleaned_data.get('importer_type') == 'SHP' and \
           not self.cleaned_data.get('zipped'):
            raise forms.ValidationError(_(u"Shapefiles must be provided in a "\
                                          u"zipped archive."))
        if self.cleaned_data.get('source') and \
           self.cleaned_data.get('source_file'):
            raise forms.ValidationError(_(u"You have to set \"source\" or "
                                          u"\"source file\" but not both."))
        if not self.cleaned_data.get('source') and \
           not self.cleaned_data.get('source_file') and \
           self.cleaned_data.get('importer_type') != 'OSM':
            raise forms.ValidationError(_(u"You have to set \"source\" or "
                                          u"\"source file\"."))
        return self.cleaned_data

class CategoryAdminForm(forms.ModelForm):
    """
    Main form for categories
    """
    description = forms.CharField(widget=TextareaAdminWidget, required=False)
    class Media:
        js = list(settings.JQUERY_JS_URLS) + [
            '%schimere/js/menu-sort.js' % settings.STATIC_URL,
        ]
    class Meta:
        model = Category

class MarkerAdminFormBase(forms.ModelForm):
    """
    Main form for marker
    """
    description = forms.CharField(widget=TextareaAdminWidget, required=False)
    class Meta:
        model = Marker

    def __init__(self, *args, **keys):
        """
        Custom initialization method in order to manage properties
        """
        self.pms = [pm for pm in PropertyModel.objects.filter(available=True)]
        if 'instance' in keys and keys['instance']:
            instance = keys['instance']
            property_dct = {}
            for pm in self.pms:
                property  = instance.getProperty(pm)
                if property:
                    property_dct[pm.getNamedId()] = property.value
            if 'initial' in keys:
                keys['initial'].update(property_dct)
            else:
                keys['initial'] = property_dct
        subcategories = keys.pop('subcategories') \
                        if 'subcategories' in keys else []
        super(MarkerAdminFormBase, self).__init__(*args, **keys)
        if settings.CHIMERE_DAYS_BEFORE_EVENT:
            self.fields['start_date'].widget = DatePickerWidget()
            self.fields['end_date'].widget = DatePickerWidget()
        if subcategories:
            self.fields['categories'].choices = subcategories

    def clean(self):
        '''
        Verify that a start date is provided when an end date is set
        Verify the mandatory properties (to be check manualy because it depends
        on the checked categories)
        '''
        start_date = self.cleaned_data.get('start_date')
        end_date = self.cleaned_data.get('end_date')
        if end_date and not start_date:
            msg = _(u"End date has been set with no start date")
            self._errors["end_date"] = self.error_class([msg])
            del self.cleaned_data['end_date']
        if end_date and start_date and start_date > end_date:
            msg = _(u"End date can't be before start date")
            self._errors["end_date"] = self.error_class([msg])
            raise forms.ValidationError(msg)
        for pm in self.pms:
            if not pm.mandatory or self.cleaned_data[pm.getNamedId()]:
                continue
            pm_cats = pm.subcategories.all()
            if not pm_cats or \
               [submited_cat for submited_cat in self.cleaned_data['categories']
                             if submited_cat in pm_cats]:
                msg = _(u"This field is mandatory for the selected categories")
                self._errors[pm.getNamedId()] = self.error_class([msg])
                #raise forms.ValidationError()
        return self.cleaned_data

    def save(self, *args, **keys):
        """
        Custom save method in order to manage associated properties
        """
        new_marker = super(MarkerAdminFormBase, self).save(*args, **keys)
        if 'status' not in self.cleaned_data and not new_marker.status:
            new_marker.status = 'S'
        if new_marker.status == 'A':
            tz = UTC()
            new_marker.available_date = datetime.replace(datetime.utcnow(),
                                                         tzinfo=tz)
        new_marker.save()
        # save properties
        properties = dict([(k.split('_')[-1], self.cleaned_data[k]) \
                for k in self.cleaned_data.keys() if k.startswith('property_')])
        new_marker.saveProperties(properties)
        return new_marker

# As we have dynamic fields, it's cleaner to make the class dynamic too
fields = {}
# declare properties
for prop in PropertyModel.objects.filter(available=True):
    key = "property_%d_%d" % (prop.order, prop.id)
    fields[key] = forms.CharField(label=prop.name,
                                 widget=PropertyModel.TYPE_WIDGET[prop.type],
                                 required=False)
MarkerAdminForm = type("MarkerAdminForm", (MarkerAdminFormBase,), fields)

class MarkerForm(MarkerAdminForm):
    """
    Form for the edit page
    """
    ref_pk = forms.IntegerField(label=u" ", widget=forms.HiddenInput(),
                                required=False)
    description = forms.CharField(widget=TextareaWidget, required=False)
    class Meta:
        model = Marker
        exclude = ('status',)
        widgets = {
            'description': TextareaWidget(),
        }

class RouteAdminForm(forms.ModelForm):
    """
    Main form for route
    """
    class Meta:
        model = Route

    def __init__(self, *args, **keys):
        """
        Custom initialization method in order to manage properties
        """
        if 'instance' in keys and keys['instance']:
            instance = keys['instance']
            property_dct = {}
            for pm in PropertyModel.objects.filter(available=True):
                property  = instance.getProperty(pm)
                if property:
                    property_dct[pm.getNamedId()] = property.value
            if 'initial' in keys:
                keys['initial'].update(property_dct)
            else:
                keys['initial'] = property_dct
        subcategories = keys.pop('subcategories') \
                        if 'subcategories' in keys else []
        super(RouteAdminForm, self).__init__(*args, **keys)
        if settings.CHIMERE_DAYS_BEFORE_EVENT:
            self.fields['start_date'].widget = DatePickerWidget()
            self.fields['end_date'].widget = DatePickerWidget()
        if subcategories:
            self.fields['categories'].choices = subcategories

    def save(self, *args, **keys):
        """
        Custom save method in order to manage associated properties
        """
        new_route = super(RouteAdminForm, self).save(*args, **keys)
        if 'status' not in self.cleaned_data and not new_route.status:
            new_route.status = 'S'
        new_route.save()
        return new_route

class RouteForm(RouteAdminForm):
    """
    Form for the edit page
    """
    description = forms.CharField(widget=TextareaWidget, required=False)
    point = forms.CharField(label=" ", required=False, widget=forms.HiddenInput)
    associated_file_id = forms.CharField(label=" ", required=False,
                                   widget=forms.HiddenInput)
    class Meta:
        model = Route
        exclude = ('status',)

    def __init__(self, *args, **kwargs):
        if kwargs.get('instance'):
            try:
                marker = Marker.objects.get(route=kwargs['instance'])
                kwargs['initial'] = {
                    'point':marker.point,
                    'description':marker.description}
                property_dct = {}
                for pm in PropertyModel.objects.filter(available=True):
                    property  = marker.getProperty(pm)
                    if property:
                        property_dct[pm.getNamedId()] = property.value
                if 'initial' in kwargs:
                    kwargs['initial'].update(property_dct)
                else:
                    kwargs['initial'] = property_dct
            except:
                pass
        super(RouteForm, self).__init__(*args, **kwargs)

    def save(self, *args, **keys):
        """
        Custom save method in order to manage associated marker and file
        """
        new_route = super(RouteForm, self).save(*args, **keys)
        if new_route.status == 'S':
            new_route.has_associated_marker = True
            new_route.save()
        # associate a route file
        if 'associated_file_id' in self.cleaned_data and \
           self.cleaned_data['associated_file_id']:
            file_pk = int(self.cleaned_data['associated_file_id'])
            new_route.associated_file = RouteFile.objects.get(pk=file_pk)
            new_route.save()
        # change the associated marker (if available)
        q_new_marker = Marker.objects.filter(route=new_route)
        if not q_new_marker.count():
            return new_route
        new_marker = q_new_marker.all()[0]
        # save description
        if self.cleaned_data['description']:
            new_marker.description = self.cleaned_data['description']
            new_marker.save()
        # save properties
        properties = dict([(k.split('_')[-1], self.cleaned_data[k]) \
                for k in self.cleaned_data.keys() if k.startswith('property_')])
        new_marker.saveProperties(properties)
        return new_route

class BaseFileForm(forms.ModelForm):
    id = forms.IntegerField(label=u"", widget=forms.HiddenInput(),
                            required=False)

    def __init__(self, *args, **kwargs):
        if not hasattr(self, '_related_name') or not self._related_name:
            raise ImproperlyConfigured
        super(BaseFileForm, self).__init__(*args, **kwargs)
        self.fields.pop('marker')

    def save(self, associated_marker):
        if not hasattr(self, 'cleaned_data') or not self.cleaned_data:
            return
        instance = None
        if self.cleaned_data.get('id'):
            try:
                instance = self._meta.model.objects.get(
                                    pk=self.cleaned_data['id'])
            except:
                pass
            self.cleaned_data.pop('id')
        if self.cleaned_data.get('DELETE'):
            if instance:
                instance.delete()
            return
        self.cleaned_data.pop('DELETE')
        self.cleaned_data['marker'] = associated_marker
        if instance:
            for k in self.cleaned_data:
                setattr(instance, k, self.cleaned_data[k])
            instance.save()
        else:
            instance = self._meta.model.objects.create(**self.cleaned_data)

class MultimediaFileAdminForm(forms.ModelForm):
    class Meta:
        model = MultimediaFile
    class Media:
        js = list(settings.JQUERY_JS_URLS) + [
            '%schimere/js/menu-sort.js' % settings.STATIC_URL,
        ]

    def __init__(self, *args, **kwargs):
        super(MultimediaFileAdminForm, self).__init__(*args, **kwargs)
        self.fields['multimedia_type'].widget.choices = \
                                                MultimediaType.get_tuples()

class MultimediaFileForm(BaseFileForm):
    """
    Form for a multimedia file
    """
    _related_name = 'multimedia_files'
    class Meta:
        model = MultimediaFile
        exclude = ('order',)

    def __init__(self, *args, **kwargs):
        super(MultimediaFileForm, self).__init__(*args, **kwargs)
        self.fields['multimedia_type'].widget.choices = \
                                                MultimediaType.get_tuples()
        # this can be auto detect
        self.fields['multimedia_type'].required = False

MultimediaFileFormSet = formset_factory(MultimediaFileForm, can_delete=True)

class PictureFileAdminForm(forms.ModelForm):
    class Meta:
        model = PictureFile
    class Media:
        js = list(settings.JQUERY_JS_URLS) + [
            '%schimere/js/menu-sort.js' % settings.STATIC_URL,
        ]

class PictureFileForm(BaseFileForm):
    """
    Form for a picture file
    """
    _related_name = 'pictures'
    class Meta:
        model = PictureFile
        exclude = ('order', 'height', 'width', 'thumbnailfile',
                   'thumbnailfile_height', 'thumbnailfile_width')

PictureFileFormSet = formset_factory(PictureFileForm, can_delete=True)

class FileForm(forms.Form):
    raw_file  = forms.FileField(label=_(u"File"))

    def clean_raw_file(self):
        data = self.cleaned_data['raw_file']
        if '.' not in data.name or \
           data.name.split('.')[-1].lower() not in ('kml', 'gpx'):
            raise forms.ValidationError(_(u"Bad file format: this must be a "\
                                          u"GPX or KML file"))
        return data

class FullFileForm(FileForm):
    name = forms.CharField(label=_(u"Name"), max_length=150)
    def __init__(self, *args, **kwargs):
        super(FullFileForm, self).__init__(*args, **kwargs)
        self.fields.keyOrder = ['name', 'raw_file']

class AreaAdminForm(forms.ModelForm):
    """
    Admin page to create an area
    """
    area = AreaField(label=_("Area"), fields=(PointField(), PointField()))
    welcome_message = forms.CharField(widget=TextareaAdminWidget,
                                      required=False)
    class Meta:
        model = Area

    def __init__(self, *args, **keys):
        """
        Custom initialization method in order to manage area
        """
        if args:
            vals = args[0]
            for k in ('upper_left_lat', 'upper_left_lon',
                      'lower_right_lat', 'lower_right_lon'):
                v = vals.get(k)
                try:
                    v = float(v)
                except ValueError:
                    v = None
                if not v:
                    args[0][k] = None
        if 'instance' in keys and keys['instance']:
            instance = keys['instance']
            dct = {'area':(instance.upper_left_corner,
                             instance.lower_right_corner)}
            if 'initial' in keys:
                keys['initial'].update(dct)
            else:
                keys['initial'] = dct
        super(AreaAdminForm, self).__init__(*args, **keys)

    def clean(self):
        '''
        Verify that the area is not empty
        '''
        if not self.cleaned_data.get('upper_left_lat') \
           and not self.cleaned_data.get('upper_left_lon') \
           and not self.cleaned_data.get('lower_right_lat') \
           and not self.cleaned_data.get('lower_right_lon') \
           and not self.cleaned_data.get('area'):
            msg = _(u"No area selected.")
            raise forms.ValidationError(msg)
        if self.cleaned_data.get('order'):
            q = Area.objects.filter(order=self.cleaned_data.get('order'))
            if self.instance:
                q = q.exclude(pk=self.instance.pk)
            if q.count():
                msg= _(u"The area \"%s\" has the same order, you need to "
                       u" choose another one.") % unicode(q.all()[0])
                raise forms.ValidationError(msg)
        return self.cleaned_data

    def save(self, *args, **keys):
        """
        Custom save method in order to manage area
        """
        new_area = super(AreaAdminForm, self).save(*args, **keys)
        area = self.cleaned_data['area']
        new_area.upper_left_corner = 'POINT(%s %s)' % (area[0][0], area[0][1])
        new_area.lower_right_corner = 'POINT(%s %s)' % (area[1][0],
                                                         area[1][1])
        content_type = ContentType.objects.get(app_label="chimere",
                                               model="area")
        if new_area.urn:
            mnemo = 'change_area_' + new_area.urn
            perm = Permission.objects.filter(codename=mnemo)
            if not perm:
                perm = Permission(name='Can change ' + new_area.name,
                           content_type_id=content_type.id, codename=mnemo)
                perm.save()
        else:
            if 'urn' in self.initial:
                mnemo = 'change_area_' + self.initial['urn']
                perm = Permission.objects.filter(codename=mnemo)
                if perm:
                    perm[0].delete()
        return new_area

class AreaForm(AreaAdminForm):
    """
    Form for the edit page
    """
    class Meta:
        model = Area




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import template
import re

register = template.Library()

def unlocalize_point(value):
    """
    Basic unlocalize filter for django 1.2
    """
    return unicode(value).replace(',', '.')

register.filter(unlocalize_point)




# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
import chimere


try:
    reqs = open(os.path.join(os.path.dirname(__file__),
                             'requirements.txt')).read()
except (IOError, OSError):
    reqs = ''

setup(
    name='chimere',
    version=chimere.get_version(),
    description=
    "Chimere is a kind of online “mashup” which is designed to aggregate "\
    "geographic data from several sources. Chimere gets a map from "\
    "OpenStreetMap and display other data added by users. Datas are freely "\
    "submitted by visitors and then validated by an administrator.",
    long_description=open('README.txt').read(),
    author=u'Étienne Loks',
    author_email='etienne.loks@peacefrogs.net',
    url='http://blog.peacefrogs.net/nim/chimere/',
    license='GPL v3 licence, see COPYING',
    packages=find_packages(),
    include_package_data=True,
    install_requires=reqs,
    #test_suite = "chimere.runtests.runtests",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
    ]
)
